#!/usr/bin/env python
"""
The cat module allows you to execute an insights datasource and write its
output to stdout. A string representation of the datasource is written to
stderr before the output.

>>> insights-cat hostname
CommandOutputProvider("/usr/bin/hostname -f")
alonzo

Pass -q if you want only the datasource information.

>>> insights-cat -q ethtool
CommandOutputProvider("/sbin/ethtool docker0")
CommandOutputProvider("/sbin/ethtool enp0s31f6")
CommandOutputProvider("/sbin/ethtool lo")
CommandOutputProvider("/sbin/ethtool tun0")
CommandOutputProvider("/sbin/ethtool virbr0")
CommandOutputProvider("/sbin/ethtool virbr0-nic")
CommandOutputProvider("/sbin/ethtool wlp3s0")
"""
from __future__ import print_function
import argparse
import logging
import os
import sys
import yaml

from contextlib import contextmanager

from insights import (apply_configs, create_context, dr, extract, HostContext,
        load_default_plugins)
from insights.core.spec_factory import ContentProvider

try:
    import colorama as C
    C.init()
except:
    class Pass(object):
        def __getattr__(self, name):
            return ""

    class C(object):
        Fore = Pass()
        Style = Pass()


def parse_args():
    p = argparse.ArgumentParser("Insights spec runner.")
    p.add_argument("-c", "--config", help="Configure components.")
    p.add_argument("-p", "--plugins", default="", help="Comma-separated list without spaces of package(s) or module(s) containing plugins.")
    p.add_argument("-q", "--quiet", action="store_true", help="Only show commands or paths.")
    p.add_argument("--no-header", action="store_true", help="Don't print command or path headers.")
    p.add_argument("-D", "--debug", action="store_true", help="Show debug level information.")
    p.add_argument("spec", nargs=1, help="Spec to dump.")
    p.add_argument("archive", nargs="?", help="Archive or directory to analyze.")
    return p.parse_args()


def configure_logging(debug):
    if debug:
        logging.basicConfig(level=logging.DEBUG, stream=sys.stderr)


def parse_plugins(raw):
    for path in raw.split(","):
        path = path.strip()
        if path.endswith(".py"):
            path, _ = os.path.splitext(path)
        path = path.rstrip("/").replace("/", ".")
        yield path


def load_plugins(raw):
    if raw:
        for p in parse_plugins(raw):
            dr.load_components(p, continue_on_error=False)


def configure(config):
    if config:
        with open(config) as f:
            apply_configs(yaml.safe_load(f))


def get_spec(fqdn):
    if "." not in fqdn:
        fqdn = "insights.specs.Specs.%s" % fqdn
    return dr.get_component(fqdn)


@contextmanager
def create_broker(root=None):
    if not root:
        broker = dr.Broker()
        broker[HostContext] = HostContext()
        yield broker
    else:
        def from_dir(d):
            broker = dr.Broker()
            ctx = create_context(d, None)
            broker[ctx.__class__] = ctx
            return broker

        if os.path.isdir(root):
            yield from_dir(root)
        else:
            with extract(root) as ex:
                yield from_dir(ex.tmp_dir)


def dump_spec(value, quiet=False, no_header=False):
    if not value:
        return

    value = value if isinstance(value, list) else [value]
    for v in value:
        if not no_header:
            vname = str(v) if isinstance(v, ContentProvider) else "Raw Data"
            print(C.Fore.BLUE + vname + C.Style.RESET_ALL, file=sys.stderr)
        if not quiet:
            if isinstance(v, ContentProvider):
                for d in v.stream():
                    print(d)
            else:
                print(v)


def dump_error(spec, broker):
    if spec in broker.exceptions:
        for ex in broker.exceptions[spec]:
            print(broker.tracebacks[ex], file=sys.stderr)

    if spec in broker.missing_requirements:
        missing = broker.missing_requirements[spec]
        required = missing[0]
        at_least_one = missing[1]

        buf = sys.stderr

        print("Missing Dependencies:", file=buf)
        if required:
            print("    Requires:", file=buf)
            for d in required:
                print("        %s" % dr.get_name(d), file=buf)
        if at_least_one:
            for alo in at_least_one:
                print("    At Least One Of:", file=buf)
                for d in alo:
                    print("        %s" % dr.get_name(d), file=buf)


def run(spec, archive=None, quiet=False, no_header=False):
    with create_broker(archive) as broker:
        value = dr.run(spec, broker=broker).get(spec)
        if value:
            dump_spec(value, quiet=quiet, no_header=no_header)
        else:
            dump_error(spec, broker)
            return sys.exit(1)


def main():
    args = parse_args()
    configure_logging(args.debug)
    load_default_plugins()
    load_plugins(args.plugins)
    configure(args.config)
    spec = get_spec(args.spec[0])
    if not spec:
        print("Spec not found: %s" % args.spec[0], file=sys.stderr)
        sys.exit(1)
    run(spec, archive=args.archive, quiet=args.quiet, no_header=args.no_header)


if __name__ == "__main__":
    main()



from insights.parsers import httpd_conf
from insights.parsers.httpd_conf import HttpdConf
from insights.tests import context_wrap
import doctest

HTTPD_CONF_1 = """
ServerRoot "/etc/httpd"
<Directory />
    Options FollowSymLinks
    AllowOverride None
</Directory>

SSLProtocol -ALL +SSLv3
#SSLProtocol all -SSLv2

NSSProtocol SSLV3 TLSV1.0
#NSSProtocol ALL

# prefork MPM
 <IfModule prefork.c>
StartServers       8
MinSpareServers    5
MaxSpareServers   20
ServerLimit      256
MaxClients       256
MaxRequestsPerChild  200
 </IfModule>

# worker MPM
<IfModule worker.c>
StartServers         4
MaxClients         300
MinSpareThreads     25
MaxSpareThreads     75
ThreadsPerChild     25
MaxRequestsPerChild  0
</IfModule>
LoadModule auth_basic_module modules/mod_auth_basic.so
LoadModule auth_digest_module modules/mod_auth_digest.so
""".strip()

HTTPD_CONF_PATH = "/etc/httpd/conf/httpd.conf"
HTTPD_CONF_D_PATH = "/etc/httpd/conf.d/default.conf"

HTTPD_CONF_D_1 = """
SSLProtocol -ALL +SSLv3
#SSLProtocol all -SSLv2

#SSLCipherSuite ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW
SSLCipherSuite ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW

# MaxClients: maximum number of server processes allowed to start
   MaxClients
""".strip()

HTTPD_CONF_SPLIT = '''
LogLevel warn
IncludeOptional conf.d/*.conf
EnableSendfile on
'''.strip()

HTTPD_CONF_MORE = '''
UserDir disable
UserDir enable bob
'''.strip()

HTTPD_CONF_NEST_1 = """
<VirtualHost 192.0.2.1>
    <Directory /var/www/example>
        Options FollowSymLinks
        AllowOverride None
    </Directory>
    <IfModule mod_php4.c>
        php_admin_flag safe_mode Off
        php_admin_value register_globals    0
        php_value magic_quotes_gpc  0
        php_value magic_quotes_runtime  0
        php_value allow_call_time_pass_reference 0
    </IfModule>
    DirectoryIndex index.php
    <IfModule mod_rewrite.c>
        RewriteEngine On
        RewriteRule .* /index.php
    </IfModule>
    <IfModule mod_rewrite.c>
        RewriteEngine Off
    </IfModule>
    DocumentRoot /var/www/example
    ServerName www.example.com
    ServerAlias admin.example.com
</VirtualHost>
""".strip()

HTTPD_CONF_NEST_2 = """
<IfModule !php5_module>
  Testphp php5_1
  <IfModule !php4_module>
    Testphp php4_1
    <Location />
        <FilesMatch ".php[45]?$">
            Order allow,deny
            Deny from all
        </FilesMatch>
        <FilesMatch ".php[45]?$">
            Order deny,allow
        </FilesMatch>
    </Location>
    Testphp php4_2
  </IfModule>
  Testphp php5_2
</IfModule>
<IfModule !php5_module>
    Testphp php5_3
    JustATest on
</IfModule>
""".strip()

HTTPD_CONF_NO_NAME_SEC = """
<RequireAll>
    AuthName "NAME Access"
    Require valid-user
</RequireAll>
""".strip()

HTTPD_CONF_DOC = '''
ServerRoot "/etc/httpd"
LoadModule auth_basic_module modules/mod_auth_basic.so
LoadModule auth_digest_module modules/mod_auth_digest.so

<Directory />
    Options FollowSymLinks
    AllowOverride None
</Directory>

<IfModule mod_mime_magic.c>
#   MIMEMagicFile /usr/share/magic.mime
    MIMEMagicFile conf/magic
</IfModule>

ErrorLog "|/usr/sbin/httplog -z /var/log/httpd/error_log.%Y-%m-%d"

SSLProtocol -ALL +SSLv3
#SSLProtocol all -SSLv2

NSSProtocol SSLV3 TLSV1.0
#NSSProtocol ALL

# prefork MPM
 <IfModule prefork.c>
StartServers       8
MinSpareServers    5
MaxSpareServers   20
ServerLimit      256
MaxClients       256
MaxRequestsPerChild  200
 </IfModule>

# worker MPM
<IfModule worker.c>
StartServers         4
MaxClients         300
MinSpareThreads     25
MaxSpareThreads     75
ThreadsPerChild     25
MaxRequestsPerChild  0
</IfModule>
'''.strip()


def test_get_httpd_conf_nest_1():
    context = context_wrap(HTTPD_CONF_NEST_1, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert result[("VirtualHost", "192.0.2.1")][("IfModule", "mod_php4.c")]['php_admin_flag'][-1].value == "safe_mode Off"
    assert result[("VirtualHost", "192.0.2.1")][("IfModule", "mod_rewrite.c")]['RewriteEngine'][-1].value == "Off"
    assert result[("VirtualHost", "192.0.2.1")][("IfModule", "mod_rewrite.c")]['RewriteRule'][-1].value == ".* /index.php"
    assert result[("VirtualHost", "192.0.2.1")]['ServerName'][-1].value == "www.example.com"


def test_get_httpd_conf_nest_2():
    context = context_wrap(HTTPD_CONF_NEST_2, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert result[("IfModule", "!php5_module")] == {
            'Testphp': [
                ('php5_1', 'Testphp php5_1', 'IfModule', '!php5_module', 'httpd.conf', '/etc/httpd/conf/httpd.conf'),
                ('php5_2', 'Testphp php5_2', 'IfModule', '!php5_module', 'httpd.conf', '/etc/httpd/conf/httpd.conf'),
                ('php5_3', 'Testphp php5_3', 'IfModule', '!php5_module', 'httpd.conf', '/etc/httpd/conf/httpd.conf')],
            'JustATest': [
                ('on', 'JustATest on', 'IfModule', '!php5_module', 'httpd.conf', '/etc/httpd/conf/httpd.conf')],
            ('IfModule', '!php4_module'): {
                ('Location', '/'): {
                    ('FilesMatch', '".php[45]?$"'): {
                        'Deny': [
                            ('from all', 'Deny from all', 'FilesMatch', '".php[45]?$"', 'httpd.conf', '/etc/httpd/conf/httpd.conf')],
                        'Order': [
                            ('allow,deny', 'Order allow,deny', 'FilesMatch', '".php[45]?$"', 'httpd.conf', '/etc/httpd/conf/httpd.conf'),
                            ('deny,allow', 'Order deny,allow', 'FilesMatch', '".php[45]?$"', 'httpd.conf', '/etc/httpd/conf/httpd.conf')]
                    }
                },
                'Testphp': [
                    ('php4_1', 'Testphp php4_1', 'IfModule', '!php4_module', 'httpd.conf', '/etc/httpd/conf/httpd.conf'),
                    ('php4_2', 'Testphp php4_2', 'IfModule', '!php4_module', 'httpd.conf', '/etc/httpd/conf/httpd.conf')]
            }
    }


def test_get_httpd_conf_1():
    context = context_wrap(HTTPD_CONF_1, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert "SSLCipherSuite" not in result
    assert result['ServerRoot'][0].value == '/etc/httpd'
    assert "SSLV3 TLSV1.0" in result["NSSProtocol"][-1]
    assert result[("IfModule", "prefork.c")]["MaxClients"][-1].value == "256"
    assert result[("IfModule", "worker.c")]["MaxClients"][-1].value == "300"
    assert type(result[("IfModule", "worker.c")]) is dict
    assert result.file_path == HTTPD_CONF_PATH
    assert 'ThreadsPerChild' not in result[('IfModule', 'prefork.c')]
    assert result[('IfModule', 'prefork.c')]['MaxRequestsPerChild'][-1].value == '200'
    assert result.file_name == "httpd.conf"
    assert result['LoadModule'][0].value == 'auth_basic_module modules/mod_auth_basic.so'
    assert result['LoadModule'][-1].value == 'auth_digest_module modules/mod_auth_digest.so'
    assert result['Directory', '/']['Options'][-1].value == 'FollowSymLinks'


def test_get_httpd_conf_2():
    context = context_wrap(HTTPD_CONF_D_1, path=HTTPD_CONF_D_PATH)
    result = HttpdConf(context)

    except_SSLC = 'ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW'
    assert result["SSLProtocol"] == [('-ALL +SSLv3', 'SSLProtocol -ALL +SSLv3', None, None, result.file_name, result.file_path)]
    assert result["SSLCipherSuite"][-1].value == except_SSLC
    assert "NSSProtocol" not in result
    assert "MaxClients" not in result
    assert result.file_path == HTTPD_CONF_D_PATH
    assert result.file_name == "default.conf"
    assert result["SSLProtocol"][-1].value == '-ALL +SSLv3'
    assert result["SSLProtocol"][-1].line == 'SSLProtocol -ALL +SSLv3'


def test_main_config_splitting():
    context = context_wrap(HTTPD_CONF_SPLIT, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert result.file_path == HTTPD_CONF_PATH
    assert result.file_name == "httpd.conf"
    assert result['LogLevel'] == [('warn', 'LogLevel warn', None, None, result.file_name, result.file_path)]
    assert result['EnableSendfile'] == [('on', 'EnableSendfile on', None, None, result.file_name, result.file_path)]
    assert result.first_half['LogLevel'][-1].value == 'warn'
    assert result.first_half['LogLevel'][-1].line == 'LogLevel warn'
    assert result.second_half['EnableSendfile'][-1].value == 'on'


def test_main_config_no_splitting():
    context = context_wrap(HTTPD_CONF_1, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert result.file_path == HTTPD_CONF_PATH
    assert result.file_name == "httpd.conf"
    assert result.data == result.first_half
    assert result.second_half == {}


def test_main_config_no_main_config():
    context = context_wrap(HTTPD_CONF_D_1, path=HTTPD_CONF_D_PATH)
    result = HttpdConf(context)

    assert result.first_half == {}
    assert result.second_half == {}


def test_multiple_values_for_directive():
    context = context_wrap(HTTPD_CONF_MORE, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert result.file_path == HTTPD_CONF_PATH
    assert result.file_name == "httpd.conf"
    assert result['UserDir'] == [
            ('disable', 'UserDir disable', None, None, result.file_name, result.file_path),
            ('enable bob', 'UserDir enable bob', None, None, result.file_name, result.file_path)]
    assert len(result['UserDir']) == 2
    assert result['UserDir'][0].value == 'disable'
    assert result['UserDir'][1].value == 'enable bob'


def test_no_name_section():
    context = context_wrap(HTTPD_CONF_NO_NAME_SEC, path=HTTPD_CONF_PATH)
    result = HttpdConf(context)

    assert result[("RequireAll", "")]["AuthName"][-1].value == "NAME Access"
    assert result[("RequireAll", "")]["Require"][-1].value == "valid-user"


def test_doc():
    env = {
            'HttpdConf': HttpdConf,
            'httpd_conf': HttpdConf(context_wrap(HTTPD_CONF_DOC, path='/path')),
          }
    failed, total = doctest.testmod(httpd_conf, globs=env)
    assert failed == 0



from insights.parsers.libvirtd_log import LibVirtdLog
from insights.tests import context_wrap

from datetime import datetime

LIBVIRTD_LOG = """
2013-10-23 17:32:18.909+0000: 14069: debug : virConnectOpen:1331 : name=qemu+tls://AA.BB.CC.DD/system
2013-10-23 17:32:18.909+0000: 14069: debug : virConnectGetConfigFile:953 : Loading config file '/etc/libvirt/libvirt.conf'
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1127 : name "qemu+tls://AA.BB.CC.DD/system" to URI components:
  scheme qemu+tls
  server AA.BB.CC.DD
  user (null)
  port 0
  path /system

2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1174 : trying driver 0 (Test) ...
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1180 : driver 0 Test returned DECLINED
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1174 : trying driver 1 (ESX) ...
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1180 : driver 1 ESX returned DECLINED
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1174 : trying driver 2 (remote) ...
2013-10-23 17:32:19.957+0000: 14069: error : virNetTLSContextCheckCertDN:418 : Certificate [session] owner does not match the hostname AA.BB.CC.DD <============= IP Address
2013-10-23 17:32:19.957+0000: 14069: warning : virNetTLSContextCheckCertificate:1102 : Certificate check failed Certificate [session] owner does not match the hostname AA.BB.CC.DD
2013-10-23 17:32:19.957+0000: 14069: error : virNetTLSContextCheckCertificate:1105 : authentication failed: Failed to verify peer's certificate

""".strip()


def test_libvirtd_log():
    log = LibVirtdLog(context_wrap(LIBVIRTD_LOG))
    assert "Certificate check failed Certificate" in log

    assert log.get("authentication failed: Failed to verify peer's certificate")[0]['raw_message'] == \
            "2013-10-23 17:32:19.957+0000: 14069: error : virNetTLSContextCheckCertificate:1105 : authentication failed: Failed to verify peer's certificate"
    # Includes continuation of URI component expansion
    assert len(list(log.get_after(datetime(2013, 10, 23, 17, 32, 19)))) == 15



# flake8: noqa
from insights.configtree.iniconfig import parse_doc


CONF1 = """
# See smb.conf.example for a more detailed config file or
# read the smb.conf manpage.
# Run 'testparm' to verify the config is correct after
# you modified it.

[global]
	workgroup = SAMBA
	security = user

	passdb backend = tdbsam

	printing = cups
	printcap name = cups
	load printers = yes
	cups options = raw

[homes]
	comment = Home Directories
	valid users = %S, %D%w%S
	browseable = No
	read only = No
	inherit acls = Yes

[printers]
	comment = All Printers
	path = /var/tmp
	printable = Yes
	create mask = 0600
	browseable = No

[print$]
	comment = Printer Drivers
	path = /var/lib/samba/drivers
	write list = @printadmin root
	force group = @printadmin
	create mask = 0664
	directory mask = 0775
""".strip().splitlines()


def test_smb_conf():
    conf = parse_doc(CONF1)
    assert conf["global"]["workgroup"][0].value == "SAMBA"



import requests

import redis
import calendar
from cachecontrol.heuristics import BaseHeuristic
from cachecontrol.wrapper import CacheControl
from cachecontrol.caches.file_cache import FileCache

from datetime import datetime, timedelta
from email.utils import parsedate, formatdate
from cachecontrol.caches.redis_cache import RedisCache


class RemoteResource(object):
    """
    RemoteResource class for accessing external Web resources.

    Attributes:
        timeout(float): Time in seconds for the requests.get api call to wait before returning a timeout exception

    Examples:
        >>> from insights.core.remote_resource import RemoteResource
        >>> rr = RemoteResource()
        >>> rtn = rr.get("http://google.com")
        >>> print (rtn.content)
    """

    timeout = 10

    def __init__(self, session=None):

        self.session = session or requests.Session()

    def get(self, url, params={}, headers={}, auth=(), certificate_path=None):
        """
        Returns the response payload from the request to the given URL.

        Args:
            url (str): The URL for the WEB API that the request is being made too.
            params (dict): Dictionary containing the query string parameters.
            headers (dict): HTTP Headers that may be needed for the request.
            auth (tuple): User ID and password for Basic Auth
            certificate_path (str): Path to the ssl certificate.

        Returns:
            response: (HttpResponse): Response object from requests.get api request
        """

        certificate_path = certificate_path if certificate_path else False
        return self.session.get(url, params=params, headers=headers, verify=certificate_path, auth=auth,
                            timeout=self.timeout)


class CachedRemoteResource(RemoteResource):
    """
    RemoteResource subclass that sets up caching for subsequent Web resource requests.

    Attributes:
        expire_after (float): Amount of time in seconds that the cache will expire
        backend (str): Type of storage for cache `DictCache1`, `FileCache` or `RedisCache`
        redis_host (str): Hostname of redis instance if `RedisCache` backend is specified
        redis_port (int): Port used to contact the redis instance if `RedisCache` backend is specified
        file_cache_path (string): Path to where file cache will be stored if `FileCache` backend is specified

    Examples:
        >>> from insights.core.remote_resource import CachedRemoteResource
        >>> crr = CachedRemoteResource()
        >>> rtn = crr.get("http://google.com")
        >>> print (rtn.content)

    """

    expire_after = 180
    backend = "DictCache"
    redis_port = 6379
    redis_host = 'localhost'
    __heuristic = 'DefaultHeuristic'
    __cache = None
    file_cache_path = '.web_cache'

    def __init__(self):

        session = requests.Session()
        hclass = globals()[self.__heuristic]

        if not self.__class__.__cache:
            if self.backend == "RedisCache":
                pool = redis.ConnectionPool(host=self.redis_host, port=self.redis_port, db=0)
                r = redis.Redis(connection_pool=pool)
                self.__class__.cache = RedisCache(r)
            elif self.backend == "FileCache":
                self.__class__.cache = FileCache(self.file_cache_path)
            else:
                self.__class__.cache = None

        session = CacheControl(session, heuristic=hclass(self.expire_after), cache=self.__class__.cache)

        super(CachedRemoteResource, self).__init__(session)


class DefaultHeuristic(BaseHeuristic):
    """
    BaseHeuristic subclass that sets the default caching headers if not supplied by the remote service.

    Attributes:
        default_cache_vars (str): Message content warning that the response from the remote server did not
          return proper HTTP cache headers so we will use default cache settings
        server_cache_headers (str): Message content warning that we are using cache settings returned by the
          remote server.
    """

    default_cache_vars = "Remote service caching headers not set correctly, using default caching"
    server_cache_headers = "Caching being done based on caching headers returned by remote service"

    def __init__(self, expire_after):

        self.expire_after = expire_after

    def update_headers(self, response):
        """
        Returns the updated caching headers.

        Args:
            response (HttpResponse): The response from the remote service

        Returns:
            response:(HttpResponse.Headers): Http caching headers
        """
        if 'expires' in response.headers and 'cache-control' in response.headers:
            self.msg = self.server_cache_headers
            return response.headers
        else:
            self.msg = self.default_cache_vars
            date = parsedate(response.headers['date'])
            expires = datetime(*date[:6]) + timedelta(0, self.expire_after)
            response.headers.update({'expires': formatdate(calendar.timegm(expires.timetuple())),
                                'cache-control': 'public'})
            return response.headers

    def warning(self, response):
        return '110 - "%s"' % self.msg



import doctest

from insights.parsers import ls_etc
from insights.tests import context_wrap


LS_ETC = """
/etc/sysconfig:
total 96
drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
-rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
-rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub

/etc/rc.d/rc3.d:
total 4
drwxr-xr-x.  2 0 0   58 Jul  6 23:32 .
drwxr-xr-x. 10 0 0 4096 Sep 16  2015 ..
lrwxrwxrwx.  1 0 0   20 Jul  6 23:32 K50netconsole -> ../init.d/netconsole
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 S10network -> ../init.d/network
lrwxrwxrwx.  1 0 0   15 Jul  6 23:32 S97rhnsd -> ../init.d/rhnsd
"""


def test_ls_etc():
    list_etc = ls_etc.LsEtc(context_wrap(LS_ETC))
    assert "/etc/sysconfig" in list_etc
    assert len(list_etc.files_of("/etc/sysconfig")) == 3
    assert list_etc.files_of("/etc/sysconfig") == ['ebtables-config', 'firewalld', 'grub']
    assert list_etc.dirs_of("/etc/sysconfig") == ['.', '..', 'cbq', 'console']
    assert list_etc.specials_of("/etc/sysconfig") == []
    assert list_etc.total_of("/etc/sysconfig") == 96
    grub = list_etc.dir_entry("/etc/sysconfig", "grub")
    assert grub is not None
    assert grub == {
        'group': '0',
        'name': 'grub',
        'links': 1,
        'perms': 'rwxrwxrwx.',
        'raw_entry': 'lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub',
        'owner': '0',
        'link': '/etc/default/grub',
        'date': 'Jul  6 23:32',
        'type': 'l',
        'size': 17,
        'dir': '/etc/sysconfig'}
    assert list_etc.files_of("/etc/rc.d/rc3.d") == ['K50netconsole',
                                                  'S10network', 'S97rhnsd']


def test_ls_etc_documentation():
    failed_count, tests = doctest.testmod(
        ls_etc,
        globs={'ls_etc': ls_etc.LsEtc(context_wrap(LS_ETC))}
    )
    assert failed_count == 0



from insights.core.spec_factory import first_of, glob_file, simple_file, head
from functools import partial
from insights.core.context import HostArchiveContext
from insights.specs import Specs

simple_file = partial(simple_file, context=HostArchiveContext)
glob_file = partial(glob_file, context=HostArchiveContext)


class InsightsArchiveSpecs(Specs):

    all_installed_rpms = glob_file("insights_commands/rpm_-qa*")
    auditctl_status = simple_file("insights_commands/auditctl_-s")
    aws_instance_type = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_aws_instance_type")
    bios_uuid = simple_file("insights_commands/dmidecode_-s_system-uuid")
    blkid = simple_file("insights_commands/blkid_-c_.dev.null")
    brctl_show = simple_file("insights_commands/brctl_show")
    ceph_df_detail = simple_file("insights_commands/ceph_df_detail_-f_json-pretty")
    ceph_health_detail = simple_file("insights_commands/ceph_health_detail_-f_json-pretty")
    ceph_insights = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_ceph_insights")
    ceph_osd_df = simple_file("insights_commands/ceph_osd_df_-f_json-pretty")
    ceph_osd_dump = simple_file("insights_commands/ceph_osd_dump_-f_json-pretty")
    ceph_osd_ec_profile_ls = simple_file("insights_commands/ceph_osd_erasure-code-profile_ls")
    ceph_osd_tree = simple_file("insights_commands/ceph_osd_tree_-f_json-pretty")
    ceph_s = simple_file("insights_commands/ceph_-s_-f_json-pretty")
    ceph_v = simple_file("insights_commands/ceph_-v")
    certificates_enddate = simple_file("insights_commands/find_.etc.origin.node_.etc.origin.master_.etc.pki_-type_f_-exec_.usr.bin.openssl_x509_-noout_-enddate_-in_-exec_echo_FileName")
    chkconfig = simple_file("insights_commands/chkconfig_--list")
    chronyc_sources = simple_file("insights_commands/chronyc_sources")
    crt = simple_file("insights_commands/find_.etc.origin.node_.etc.origin.master_-type_f_-path_.crt")
    date = simple_file("insights_commands/date")
    date_iso = simple_file("insights_commands/date_--iso-8601_seconds")
    date_utc = simple_file("insights_commands/date_--utc")
    df__al = simple_file("insights_commands/df_-al")
    df__alP = simple_file("insights_commands/df_-alP")
    df__li = simple_file("insights_commands/df_-li")
    dig = simple_file("insights_commands/dig_dnssec_._DNSKEY")
    dig_dnssec = simple_file("insights_commands/dig_dnssec_._SOA")
    dig_edns = simple_file("insights_commands/dig_edns_0_._SOA")
    dig_noedns = simple_file("insights_commands/dig_noedns_._SOA")
    display_java = simple_file("insights_commands/alternatives_--display_java")
    display_name = simple_file("display_name")
    dmesg = simple_file("insights_commands/dmesg")
    dmidecode = simple_file("insights_commands/dmidecode")
    dmsetup_info = simple_file("insights_commands/dmsetup_info_-C")
    docker_info = simple_file("insights_commands/docker_info")
    docker_list_containers = simple_file("insights_commands/docker_ps_--all_--no-trunc")
    docker_list_images = simple_file("insights_commands/docker_images_--all_--no-trunc_--digests")
    dumpdev = simple_file("insights_commands/awk_.ext_234_._print_1_.proc.mounts")
    engine_config_all = simple_file("insights_commands/engine-config_--all")
    ethtool = glob_file("insights_commands/ethtool_*", ignore="ethtool_-.*")
    ethtool_S = glob_file("insights_commands/ethtool_-S_*")
    ethtool_a = glob_file("insights_commands/ethtool_-a_*")
    ethtool_c = glob_file("insights_commands/ethtool_-c_*")
    ethtool_g = glob_file("insights_commands/ethtool_-g_*")
    ethtool_i = glob_file("insights_commands/ethtool_-i_*")
    ethtool_k = glob_file("insights_commands/ethtool_-k_*")
    facter = simple_file("insights_commands/facter")
    fc_match = simple_file("insights_commands/fc-match_-sv_sans_regular_roman_family_fontformat")
    fdisk_l = simple_file("insights_commands/fdisk_-l")
    foreman_rake_db_migrate_status = simple_file('insights_commands/foreman-rake_db_migrate_status')
    getcert_list = simple_file("insights_commands/getcert_list")
    getenforce = simple_file("insights_commands/getenforce")
    getsebool = simple_file("insights_commands/getsebool_-a")
    grub1_config_perms = simple_file("insights_commands/ls_-l_.boot.grub.grub.conf")
    grub_config_perms = simple_file("insights_commands/ls_-l_.boot.grub2.grub.cfg")
    grubby_default_index = simple_file("insights_commands/grubby_--default-index")
    grubby_default_kernel = simple_file("insights_commands/grubby_--default-kernel")
    gluster_v_info = simple_file("insights_commands/gluster_volume_info")
    gluster_v_status = simple_file("insights_commands/gluster_volume_status")
    hammer_ping = simple_file("insights_commands/hammer_ping")
    hammer_task_list = simple_file("insights_commands/hammer_--csv_task_list")
    heat_crontab = simple_file("insights_commands/crontab_-l_-u_heat")
    heat_crontab_container = simple_file("insights_commands/docker_exec_heat_api_cron_.usr.bin.crontab_-l_-u_heat")
    installed_rpms = head(all_installed_rpms)
    hostname = first_of([simple_file("insights_commands/hostname_-f"), simple_file("insights_commands/hostname")])
    hponcfg_g = simple_file("insights_commands/hponcfg_-g")
    httpd_M = glob_file("insights_commands/*httpd*_-M")
    httpd_on_nfs = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_httpd_on_nfs")
    httpd_pid = simple_file("insights_commands/pgrep_-o_httpd")
    httpd_V = glob_file("insights_commands/*httpd*_-V")
    ifconfig = simple_file("insights_commands/ifconfig_-a")
    ip6tables = simple_file("insights_commands/ip6tables-save")
    ip_addr = simple_file("insights_commands/ip_addr")
    ip_addresses = simple_file("insights_commands/hostname_-I")
    ip_route_show_table_all = simple_file("insights_commands/ip_route_show_table_all")
    ip_s_link = simple_file("insights_commands/ip_-s_link")
    ipcs_m = simple_file("insights_commands/ipcs_-m")
    ipcs_m_p = simple_file("insights_commands/ipcs_-m_-p")
    ipcs_s = simple_file("insights_commands/ipcs_-s")
    iptables = simple_file("insights_commands/iptables-save")
    ipv4_neigh = simple_file("insights_commands/ip_-4_neighbor_show_nud_all")
    ipv6_neigh = simple_file("insights_commands/ip_-6_neighbor_show_nud_all")
    ip_netns_exec_namespace_lsof = glob_file("insights_commands/ip_netns_exec_*_lsof_-i")
    iscsiadm_m_session = simple_file("insights_commands/iscsiadm_-m_session")
    katello_service_status = simple_file("insights_commands/katello-service_status")
    keystone_crontab = simple_file("insights_commands/crontab_-l_-u_keystone")
    keystone_crontab_container = simple_file("insights_commands/docker_exec_keystone_cron_.usr.bin.crontab_-l_-u_keystone")
    libkeyutils = simple_file("insights_commands/find_-L_.lib_.lib64_-name_libkeyutils.so")
    libkeyutils_objdumps = simple_file("insights_commands/find_-L_.lib_.lib64_-name_libkeyutils.so.1_-exec_objdump_-x")
    locale = simple_file("insights_commands/locale")
    localtime = simple_file("insights_commands/file_-L_.etc.localtime")
    lpstat_p = simple_file("insights_commands/lpstat_-p")
    ls_boot = simple_file("insights_commands/ls_-lanR_.boot")
    ls_dev = simple_file("insights_commands/ls_-lanR_.dev")
    ls_disk = simple_file("insights_commands/ls_-lanR_.dev.disk")
    ls_docker_volumes = simple_file("insights_commands/ls_-lanR_.var.lib.docker.volumes")
    ls_etc = simple_file("insights_commands/ls_-lanR_.etc")
    ls_ocp_cni_openshift_sdn = simple_file("insights_commands/ls_-l_.var.lib.cni.networks.openshift-sdn")
    ls_origin_local_volumes_pods = simple_file("insights_commands/ls_-l_.var.lib.origin.openshift.local.volumes.pods")
    ls_sys_firmware = simple_file("insights_commands/ls_-lanR_.sys.firmware")
    ls_var_lib_mongodb = simple_file("insights_commands/ls_-la_.var.lib.mongodb")
    ls_R_var_lib_nova_instances = simple_file("insights_commands/ls_-laR_.var.lib.nova.instances")
    ls_var_lib_nova_instances = simple_file("insights_commands/ls_-laRZ_.var.lib.nova.instances")
    ls_var_opt_mssql = simple_file("insights_commands/ls_-ld_.var.opt.mssql")
    ls_usr_sbin = simple_file("insights_commands/ls_-ln_.usr.sbin")
    ls_var_log = simple_file("insights_commands/ls_-la_.var.log_.var.log.audit")
    ls_var_opt_mssql_log = simple_file("insights_commands/ls_-la_.var.opt.mssql.log")
    ls_var_www = simple_file("insights_commands/ls_-la_.dev.null_.var.www")
    ls_var_spool_clientmq = simple_file("insights_commands/ls_-ln_.var.spool.clientmqueue")
    ls_var_tmp = simple_file("insights_commands/ls_-ln_.var.tmp")
    ls_var_run = simple_file("insights_commands/ls_-lnL_.var.run")
    ls_var_spool_postfix_maildrop = simple_file("insights_commands/ls_-ln_.var.spool.postfix.maildrop")
    ls_osroot = simple_file("insights_commands/ls_-lan")
    lsblk = simple_file("insights_commands/lsblk")
    lsblk_pairs = simple_file("insights_commands/lsblk_-P_-o_NAME_KNAME_MAJ_MIN_FSTYPE_MOUNTPOINT_LABEL_UUID_RA_RO_RM_MODEL_SIZE_STATE_OWNER_GROUP_MODE_ALIGNMENT_MIN-IO_OPT-IO_PHY-SEC_LOG-SEC_ROTA_SCHED_RQ-SIZE_TYPE_DISC-ALN_DISC-GRAN_DISC-MAX_DISC-ZERO")
    lscpu = simple_file("insights_commands/lscpu")
    ls_lib_firmware = simple_file("insights_commands/ls_-lanR_.lib.firmware")
    lsmod = simple_file("insights_commands/lsmod")
    lsof = simple_file("insights_commands/lsof")
    lspci = simple_file("insights_commands/lspci_-k")
    lssap = simple_file("insights_commands/usr.sap.hostctrl.exe.lssap")
    lsscsi = simple_file("insights_commands/lsscsi")
    lvdisplay = simple_file("insights_commands/lvdisplay")
    lvs_noheadings = simple_file("insights_commands/lvs_--nameprefixes_--noheadings_--separator_-a_-o_lv_name_lv_size_lv_attr_mirror_log_vg_name_devices_region_size_data_percent_metadata_percent_segtype_seg_monitor_--config_global_locking_type_0")
    lvs_noheadings_all = simple_file("insights_commands/lvs_--nameprefixes_--noheadings_--separator_-a_-o_lv_name_lv_size_lv_attr_mirror_log_vg_name_devices_region_size_data_percent_metadata_percent_segtype_--config_global_locking_type_0_devices_filter_a")
    max_uid = simple_file("insights_commands/awk_-F_if_3_max_max_3_END_print_max_.etc.passwd")
    md5chk_files = simple_file("insights_commands/md5sum_.dev.null_.etc.pki._product_product-default_.69.pem")
    mlx4_port = simple_file("insights_commands/find_.sys.bus.pci.devices._.mlx4_port_0-9_-print_-exec_cat")
    mount = simple_file("insights_commands/mount")
    modinfo_i40e = simple_file("insights_commands/modinfo_i40e")
    modinfo_vmxnet3 = simple_file("insights_commands/modinfo_vmxnet3")
    multicast_querier = simple_file("insights_commands/find_.sys.devices.virtual.net._-name_multicast_querier_-print_-exec_cat")
    multipath_conf_initramfs = simple_file("insights_commands/lsinitrd_-f_.etc.multipath.conf")
    multipath__v4__ll = simple_file("insights_commands/multipath_-v4_-ll")
    mysqladmin_vars = simple_file("insights_commands/mysqladmin_variables")
    named_checkconf_p = simple_file("insights_commands/named-checkconf_-p")
    namespace = simple_file("insights_commands/ls_.var.run.netns")
    netstat = simple_file("insights_commands/netstat_-neopa")
    netstat_agn = simple_file("insights_commands/netstat_-agn")
    netstat_i = simple_file("insights_commands/netstat_-i")
    netstat_s = simple_file("insights_commands/netstat_-s")
    nmcli_conn_show = simple_file("insights_commands/nmcli_conn_show")
    nmcli_dev_show = simple_file("insights_commands/nmcli_dev_show")
    nova_crontab = simple_file("insights_commands/crontab_-l_-u_nova")
    nova_crontab_container = simple_file("insights_commands/docker_exec_nova_api_cron_.usr.bin.crontab_-l_-u_nova")
    nova_uid = simple_file("insights_commands/id_-u_nova")
    nova_migration_uid = simple_file("insights_commands/id_-u_nova_migration")
    ntpq_leap = simple_file("insights_commands/ntpq_-c_rv_0_leap")
    ntpq_pn = simple_file("insights_commands/ntpq_-pn")
    ntptime = simple_file("insights_commands/ntptime")
    numeric_user_group_name = simple_file("insights_commands/grep_-c_digit_.etc.passwd_.etc.group")
    oc_get_bc = simple_file("insights_commands/oc_get_bc_-o_yaml_--all-namespaces")
    oc_get_build = simple_file("insights_commands/oc_get_build_-o_yaml_--all-namespaces")
    oc_get_clusterrole_with_config = simple_file("insights_commands/oc_get_clusterrole_--config_.etc.origin.master.admin.kubeconfig")
    oc_get_clusterrolebinding_with_config = simple_file("insights_commands/oc_get_clusterrolebinding_--config_.etc.origin.master.admin.kubeconfig")
    oc_get_dc = simple_file("insights_commands/oc_get_dc_-o_yaml_--all-namespaces")
    oc_get_egressnetworkpolicy = simple_file("insights_commands/oc_get_egressnetworkpolicy_-o_yaml_--all-namespaces")
    oc_get_endpoints = simple_file("insights_commands/oc_get_endpoints_-o_yaml_--all-namespaces")
    oc_get_event = simple_file("insights_commands/oc_get_event_-o_yaml_--all-namespaces")
    oc_get_node = simple_file("insights_commands/oc_get_nodes_-o_yaml")
    oc_get_pod = simple_file("insights_commands/oc_get_pod_-o_yaml_--all-namespaces")
    oc_get_project = simple_file("insights_commands/oc_get_project_-o_yaml_--all-namespaces")
    oc_get_pv = simple_file("insights_commands/oc_get_pv_-o_yaml_--all-namespaces")
    oc_get_pvc = simple_file("insights_commands/oc_get_pvc_-o_yaml_--all-namespaces")
    oc_get_rc = simple_file("insights_commands/oc_get_rc_-o_yaml_--all-namespaces")
    oc_get_role = simple_file("insights_commands/oc_get_role_-o_yaml_--all-namespaces")
    oc_get_rolebinding = simple_file("insights_commands/oc_get_rolebinding_-o_yaml_--all-namespaces")
    oc_get_route = simple_file("insights_commands/oc_get_route_-o_yaml_--all-namespaces")
    oc_get_service = simple_file("insights_commands/oc_get_service_-o_yaml_--all-namespaces")
    oc_get_configmap = simple_file("insights_commands/oc_get_configmap_-o_yaml_--all-namespaces")
    openvswitch_other_config = simple_file("insights_commands/ovs-vsctl_-t_5_get_Open_vSwitch_._other_config")
    ovs_appctl_fdb_show_bridge = glob_file("insights_commands/ovs-appctl_fdb.show_*")
    ovs_vsctl_list_bridge = simple_file("insights_commands/ovs-vsctl_list_bridge")
    ovs_vsctl_show = simple_file("insights_commands/ovs-vsctl_show")
    parted__l = simple_file("insights_commands/parted_-l_-s")
    passenger_status = simple_file("insights_commands/passenger-status")
    pcs_config = simple_file("insights_commands/pcs_config")
    pcs_status = simple_file("insights_commands/pcs_status")
    ps_aux = simple_file("insights_commands/ps_aux")
    ps_auxcww = simple_file("insights_commands/ps_auxcww")
    ps_auxww = simple_file("insights_commands/ps_auxww")
    ps_ef = simple_file("insights_commands/ps_-ef")
    ps_eo = simple_file("insights_commands/ps_-eo_pid_ppid_comm")
    pvs = simple_file("insights_commands/pvs_-a_-v_-o_pv_mda_free_pv_mda_size_pv_mda_count_pv_mda_used_count_pe_count_--config_global_locking_type_0")
    pvs_noheadings = simple_file("insights_commands/pvs_--nameprefixes_--noheadings_--separator_-a_-o_pv_all_vg_name_--config_global_locking_type_0")
    pvs_noheadings_all = simple_file("insights_commands/pvs_--nameprefixes_--noheadings_--separator_-a_-o_pv_all_vg_name_--config_global_locking_type_0_devices_filter_a")
    qpid_stat_g = simple_file("insights_commands/qpid-stat_-g_--ssl-certificate_.etc.pki.katello.qpid_client_striped.crt_-b_amqps_..localhost_5671")
    qpid_stat_q = simple_file("insights_commands/qpid-stat_-q_--ssl-certificate_.etc.pki.katello.qpid_client_striped.crt_-b_amqps_..localhost_5671")
    qpid_stat_u = simple_file("insights_commands/qpid-stat_-u_--ssl-certificate_.etc.pki.katello.qpid_client_striped.crt_-b_amqps_..localhost_5671")
    rabbitmq_policies = simple_file("insights_commands/rabbitmqctl_list_policies")
    rabbitmq_queues = simple_file("insights_commands/rabbitmqctl_list_queues_name_messages_consumers_auto_delete")
    rabbitmq_report = simple_file("insights_commands/rabbitmqctl_report")
    rabbitmq_users = simple_file("insights_commands/rabbitmqctl_list_users")
    rhn_charsets = simple_file("insights_commands/rhn-charsets")
    rhn_schema_stats = simple_file("insights_commands/rhn-schema-stats")
    rhn_schema_version = simple_file("insights_commands/rhn-schema-version")
    rhev_data_center = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_rhev_data_center")
    rhv_log_collector_analyzer = simple_file("insights_commands/rhv-log-collector-analyzer_--json")
    root_crontab = simple_file("insights_commands/crontab_-l_-u_root")
    route = simple_file("insights_commands/route_-n")
    rpm_V_packages = simple_file("insights_commands/rpm_-V_coreutils_procps_procps-ng_shadow-utils_passwd_sudo")
    sapcontrol_getsystemupdatelist = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_sapcontrol_getsystemupdatelist")
    saphostctl_getcimobject_sapinstance = simple_file("insights_commands/usr.sap.hostctrl.exe.saphostctrl_-function_GetCIMObject_-enuminstances_SAPInstance")
    saphostexec_status = simple_file("insights_commands/usr.sap.hostctrl.exe.saphostexec_-status")
    saphostexec_version = simple_file("insights_commands/usr.sap.hostctrl.exe.saphostexec_-version")
    sestatus = simple_file("insights_commands/sestatus_-b")
    smbstatus_S = simple_file("insights_commands/smbstatus_-S")
    smbstatus_p = simple_file("insights_commands/smbstatus_-p")
    software_collections_list = simple_file('insights_commands/scl_--list')
    ss = simple_file("insights_commands/ss_-tupna")
    sshd_config_perms = simple_file("insights_commands/ls_-l_.etc.ssh.sshd_config")
    subscription_manager_facts_list = simple_file("insights_commands/subscription-manager_facts_--list")
    subscription_manager_id = simple_file("insights_commands/subscription-manager_identity")
    subscription_manager_list_consumed = simple_file('insights_commands/subscription-manager_list_--consumed')
    subscription_manager_list_installed = simple_file('insights_commands/subscription-manager_list_--installed')
    subscription_manager_release_show = simple_file('insights_commands/subscription-manager_release_--show')
    subscription_manager_repos_list_enabled = simple_file('insights_commands/subscription-manager_repos_--list-enabled')
    sysctl = simple_file("insights_commands/sysctl_-a")
    sysctl_conf_initramfs = simple_file("insights_commands/lsinitrd_.boot.initramfs-_kdump.img_-f_.etc.sysctl.conf_.etc.sysctl.d._.conf")
    systemctl_cinder_volume = simple_file("insights_commands/systemctl_show_openstack-cinder-volume")
    systemctl_httpd = simple_file("insights_commands/systemctl_show_httpd")
    systemctl_list_unit_files = simple_file("insights_commands/systemctl_list-unit-files")
    systemctl_list_units = simple_file("insights_commands/systemctl_list-units")
    systemctl_mariadb = simple_file("insights_commands/systemctl_show_mariadb")
    systemctl_pulp_workers = simple_file("insights_commands/systemctl_show_pulp_workers")
    systemctl_pulp_resmg = simple_file("insights_commands/systemctl_show_pulp_resource_manager")
    systemctl_pulp_celerybeat = simple_file("insights_commands/systemctl_show_pulp_celerybeat")
    systemctl_qpidd = simple_file("insights_commands/systemctl_show_qpidd")
    systemctl_qdrouterd = simple_file("insights_commands/systemctl_show_qdrouterd")
    systemctl_smartpdc = simple_file("insights_commands/systemctl_show_smart_proxy_dynflow_core")
    systool_b_scsi_v = simple_file("insights_commands/systool_-b_scsi_-v")
    teamdctl_config_dump = glob_file("insights_commands/teamdctl_*_config_dump")
    teamdctl_state_dump = glob_file("insights_commands/teamdctl_*_state_dump")
    tomcat_vdc_fallback = simple_file("insights_commands/find_.usr.share_-maxdepth_1_-name_tomcat_-exec_.bin.grep_-R_-s_VirtualDirContext_--include_.xml")
    tuned_adm = simple_file("insights_commands/tuned-adm_list")
    ulimit_hard = simple_file("insights_commands/ulimit_-a_-H")
    uname = simple_file("insights_commands/uname_-a")
    uptime = simple_file("insights_commands/uptime")
    vgdisplay = simple_file("insights_commands/vgdisplay")
    vgs_noheadings = simple_file("insights_commands/vgs_--nameprefixes_--noheadings_--separator_-a_-o_vg_all_--config_global_locking_type_0")
    vgs_noheadings_all = simple_file("insights_commands/vgs_--nameprefixes_--noheadings_--separator_-a_-o_vg_all_--config_global_locking_type_0_devices_filter_a")
    virsh_list_all = simple_file("insights_commands/virsh_--readonly_list_--all")
    virt_what = simple_file("insights_commands/virt-what")
    woopsie = simple_file("insights_commands/find_.var.crash_.var.tmp_-path_.reports-_.whoopsie-report")
    yum_repolist = simple_file("insights_commands/yum_-C_repolist")



from insights.parsers.lvm import Vgs, VgsHeadings
from insights.tests import context_wrap

VGS_INFO = """
LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='YCpusB-LEly-THGL-YXhC-t3q6-mUQV-wyFZrx'|LVM2_VG_NAME='rhel'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='476.45g'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCKTYPE=''|LVM2_VG_LOCKARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='121971'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='4'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='1020.00k'|LVM2_VG_MDA_COPIES='unmanaged'
LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='123456-LEly-THGL-YXhC-t3q6-mUQV-123456'|LVM2_VG_NAME='fedora'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='476.45g'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCKTYPE=''|LVM2_VG_LOCKARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='121971'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='4'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='1020.00k'|LVM2_VG_MDA_COPIES='unmanaged'

""".strip()

VGS_INFO_FEDORA = {
    'VG': 'fedora',
    'Attr': 'wz--n-',
    'VSize': '476.45g',
    'VFree': '4.00m',
    '#PV': '1',
    '#LV': '3',
    '#SN': '0',
    '#Ext': '121971',
    '#VMda': '1',
    '#VMdaCps': 'unmanaged',
    '#VMdaUse': '1',
    'AllocPol': 'normal',
    'Clustered': '',
    'Exported': '',
    'Ext': '4.00m',
    'Extendable': 'extendable',
    'Fmt': 'lvm2',
    'Free': '1',
    'Lock Args': '',
    'Lock_Type': '',
    'MaxLV': '0',
    'MaxPV': '0',
    'Partial': '',
    'SYS_ID': '',
    'Seq': '4',
    'System_ID': '',
    'VG_Tags': '',
    'VG_UUID': '123456-LEly-THGL-YXhC-t3q6-mUQV-123456',
    'VMdaFree': '0',
    'VMdaSize': '1020.00k',
    'VPerms': 'writeable',
    'VProfile': ''
}

VGS_HEADER_INFO = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
    Using volume group(s) on command line.
  VG            Attr   Ext   #PV #LV #SN VSize   VFree    VG UUID                                VProfile #VMda VMdaFree  VMdaSize  #VMdaUse VG Tags
  DATA_OTM_VG   wz--n- 4.00m   6   1   0   2.05t 1020.00m xK6HXk-xl2O-cqW5-2izb-LI9M-4fV0-dAzfcc              6   507.00k  1020.00k        6
  ITM_VG        wz--n- 4.00m   1   1   0  16.00g    4.00m nws5dd-INe6-1db6-9U1N-F0G3-S1z2-5XTdO4              1   508.00k  1020.00k        1
  ORABIN_OTM_VG wz--n- 4.00m   2   3   0 190.00g       0  hfJwg8-hset-YgUY-X6NJ-gkWE-EunZ-KuCXGP              2   507.50k  1020.00k        2
  REDO_OTM_VG   wz--n- 4.00m   1   3   0  50.00g       0  Q2YtGy-CWKU-sEYj-mqHk-rbdP-Hzup-wi8jsf              1   507.50k  1020.00k        1
  SWAP_OTM_VG   wz--n- 4.00m   1   1   0  24.00g    8.00g hAerzZ-U8QU-ICkc-xxCj-N2Ny-rWzq-pmTpWJ              1   508.00k  1020.00k        1
  rootvg        wz--n- 4.00m   1   6   0  19.51g    1.95g p4tLLb-ikeo-Ankk-2xJ6-iHYf-D4E6-KFCFvr              1   506.50k  1020.00k        1
    Reloading config files
    Wiping internal VG cache
""".strip()   # noqa: W291

VGS_HEADER_5 = {
    'VG': 'rootvg',
    'Attr': 'wz--n-',
    'Ext': '4.00m',
    '#PV': '1',
    '#LV': '6',
    '#SN': '0',
    'VSize': '19.51g',
    'VFree': '1.95g',
    'VG_UUID': 'p4tLLb-ikeo-Ankk-2xJ6-iHYf-D4E6-KFCFvr',
    'VProfile': '',
    '#VMda': '1',
    'VMdaFree': '506.50k',
    'VMdaSize': '1020.00k',
    '#VMdaUse': '1',
    'VG_Tags': ''
}


def test_vgs():
    vgs_records = Vgs(context_wrap(VGS_INFO))
    assert len(list(vgs_records)) == 2
    for k, v in VGS_INFO_FEDORA.items():
        assert vgs_records["fedora"][k] == v
    assert vgs_records["fedora"]['LVM2_VG_SEQNO'] == '4'


def test_vgs_headers():
    vgs_info = VgsHeadings(context_wrap(VGS_HEADER_INFO))
    assert vgs_info is not None
    assert len(vgs_info.data) == 6
    for k, v in VGS_HEADER_5.items():
        assert vgs_info[5][k] == v
    assert vgs_info[5]['VPerms'] is None



import pytest
from insights.parsers.hostname import Hostname
from insights.tests import context_wrap

HOSTNAME = "rhel7.example.com"
HOSTNAME_MULTILINE = """
rhel7.example.com
"""

HOSTNAME_SHORT = "rhel7"
HOSTNAME_SHORT_MULTILINE = """
rhel7
"""


def test_full_hostname():
    data = Hostname(context_wrap(HOSTNAME))
    assert data.fqdn == "rhel7.example.com"
    assert data.hostname == "rhel7"
    assert data.domain == "example.com"
    assert "{0}".format(data) == "<hostname: rhel7, domain: example.com>"


def test_full_multiline_hostname():
    data = Hostname(context_wrap(HOSTNAME_MULTILINE, strip=False))
    assert data.fqdn == "rhel7.example.com"
    assert data.hostname == "rhel7"
    assert data.domain == "example.com"
    assert "{0}".format(data) == "<hostname: rhel7, domain: example.com>"


def test_short_hostname():
    data = Hostname(context_wrap(HOSTNAME_SHORT))
    assert data.fqdn == "rhel7"
    assert data.hostname == "rhel7"
    assert data.domain == ""


def test_short_multiline_hostname():
    data = Hostname(context_wrap(HOSTNAME_SHORT_MULTILINE, strip=False))
    assert data.fqdn == "rhel7"
    assert data.hostname == "rhel7"
    assert data.domain == ""


def test_empty_hostname():
    with pytest.raises(Exception):
        Hostname(context_wrap(""))



from insights.configtree import startswith, istartswith
from insights.configtree import endswith, iendswith
from insights.configtree import contains, icontains
from insights.configtree import eq, ieq, le, ile, lt, ilt, ge, ige, gt, igt
from insights.configtree import first, last  # noqa: F401
from insights.combiners.httpd_conf import _HttpdConf, HttpdConfTree
from insights.combiners.httpd_conf import in_network, is_private
from insights.tests import context_wrap


HTTPD_CONF_1 = '''
JustFotTest_NoSec "/var/www/cgi"
# prefork MPM
<IfModule prefork.c>
ServerLimit      256
ThreadsPerChild  16
JustForTest      "AB"
MaxClients       256
</IfModule>

IncludeOptional conf.d/*.conf
'''.strip()

HTTPD_CONF_2 = '''
JustForTest_NoSec "/var/www/cgi"
# prefork MPM
<IfModule prefork.c>
ServerLimit      1024
JustForTest      "ABC"
MaxClients       1024
</IfModule>
'''.strip()

HTTPD_CONF_3 = '''
# prefork MPM
<IfModule prefork.c>
ServerLimit      256
MaxClients       512
</IfModule>
'''.strip()

HTTPD_CONF_SHADOWTEST_1 = '''
Foo 1A
Foo 1B
Foo 1C
<IfModule prefork.c>
Foo 1xA
Foo 1xB
Foo 1xC
Bar 1A
Bar 1B
Bar 1C
</IfModule>

IncludeOptional conf.d/*.conf
'''.strip()

HTTPD_CONF_SHADOWTEST_2 = '''
Foo 2A
Foo 2B
Foo 2C
<IfModule ASDF.prefork.c.ASDF>
Foo 2xA
Foo 2xB
Foo 2xC
Bar 2A
Bar 2B
Bar 2C
</IfModule>
'''.strip()

HTTPD_CONF_SHADOWTEST_3 = '''
Foo 3A
Foo 3B
Foo 3C
<IfModule prefork.c>
Foo 3xA
Foo 3xB
Foo 3xC
Bar 3A
Bar 3B
Bar 3C
</IfModule>
'''.strip()

HTTPD_CONF_MAIN_1 = '''
ServerRoot "/etc/httpd"
Listen 80

# Load config files in the "/etc/httpd/conf.d" directory, if any.
IncludeOptional conf.d/*.conf
'''.strip()

HTTPD_CONF_MAIN_2 = '''
# Load config files in the "/etc/httpd/conf.d" directory, if any.
IncludeOptional conf.d/*.conf

ServerRoot "/etc/httpd"
Listen 80
'''.strip()

HTTPD_CONF_MAIN_3 = '''
ServerRoot "/etc/httpd"

# Load config files in the "/etc/httpd/conf.d" directory, if any.
IncludeOptional conf.d/*.conf

Listen 80
'''.strip()

HTTPD_CONF_FILE_1 = '''
ServerRoot "/home/skontar/httpd"
Listen 8080
'''.strip()

HTTPD_CONF_FILE_2 = '''
ServerRoot "/home/skontar/www"
'''.strip()

HTTPD_CONF_MORE = '''
UserDir disable
UserDir enable bob
'''.strip()

HTTPD_CONF_NEST_1 = """
<VirtualHost 128.39.140.28>
    <Directory /var/www/example>
        Options FollowSymLinks
        AllowOverride None
    </Directory>
    <IfModule mod_php4.c>
        php_admin_flag safe_mode Off
        php_admin_value register_globals    0
    </IfModule>
    DirectoryIndex index.php
    <IfModule mod_rewrite.c>
        RewriteEngine On
        RewriteRule .* /index.php
    </IfModule>
    <IfModule mod_rewrite.c>
        RewriteEngine Off
    </IfModule>
    <IfModule !php5_module>
        <IfModule !php4_module>
            <FilesMatch ".php[45]?$">
                Order allow,deny
                Deny from all
            </FilesMatch>
            <FilesMatch ".php[45]?$">
                Order deny,allow
            </FilesMatch>
        </IfModule>
    </IfModule>
    DocumentRoot /var/www/example
    ServerName www.example.com
    ServerAlias admin.example.com
</VirtualHost>
<IfModule !php5_module>
  <IfModule !php4_module>
    <Location />
        <FilesMatch ".php[45]">
            Order allow,deny
            Deny from all
        </FilesMatch>
    </Location>
  </IfModule>
</IfModule>
<IfModule mod_rewrite.c>
    RewriteEngine Off
</IfModule>
LogLevel warn
DocumentRoot "/var/www/html_cgi"
IncludeOptional conf.d/*.conf
EnableSendfile on
""".strip()

HTTPD_CONF_NEST_2 = """
DocumentRoot "/var/www/html"
<VirtualHost 128.39.140.30>
    <IfModule !php5_module>
        <IfModule !php4_module>
            <FilesMatch ".php[45]?$">
                Order allow,deny
                Deny from all
            </FilesMatch>
            <FilesMatch ".php[45]?$">
                Order deny,allow
            </FilesMatch>
        </IfModule>
    </IfModule>
    DocumentRoot /var/www/example1
    ServerName www.example1.com
    ServerAlias admin.example1.com
</VirtualHost>
<IfModule !php5_module>
  <IfModule !php4_module>
    <Location />
        <FilesMatch test>
            Order deny,allow
            Allow from all
        </FilesMatch>
        <FilesMatch ".php[45]">
            Order deny,allow
        </FilesMatch>
    </Location>
  </IfModule>
</IfModule>
<IfModule mod_rewrite.c>
    RewriteEngine On
</IfModule>
EnableSendfile off
""".strip()

HTTPD_CONF_NEST_3 = """
<VirtualHost 128.39.140.28>
    <IfModule !php5_module>
        Testphp php5_v3_1
        <IfModule !php4_module>
            Testphp php4_v3_1
        </IfModule>
        Testphp php5_v3_2
    </IfModule>
</VirtualHost>
<IfModule !php5_module>
  Testphp php5_3_a
  <IfModule !php4_module>
    Testphp php4_3_a
  </IfModule>
</IfModule>
""".strip()

HTTPD_CONF_NEST_4 = """
<VirtualHost 128.39.140.30>
    <IfModule !php5_module>
        Testphp php5_v4_1
        <IfModule !php4_module>
            Testphp php4_v4_1
        </IfModule>
        Testphp php5_v4_2
    </IfModule>
</VirtualHost>
<IfModule !php5_module>
  Testphp php5_4_b
  <IfModule !php4_module>
    Testphp php4_4_b
  </IfModule>
</IfModule>
""".strip()


def test_startswith():
    data = ["abc", "abrd", "ed"]
    assert startswith("ab")(data)
    assert istartswith("AB")(data)
    assert startswith("ab")("abcde")
    assert not startswith("de")(data)


def test_endswith():
    data = ["abc", "abrd", "ed"]
    assert endswith("d")(data)
    assert iendswith("D")(data)
    assert endswith("d")("end")
    assert not endswith("re")(data)


def test_contains():
    data = ["abc", "abrd", "ed"]
    assert contains("b")(data)
    assert icontains("B")(data)
    assert contains("b")("abc")
    assert not contains("x")(data)


def test_equals():
    data = ["abc", "abrd", "ed"]
    assert eq("abc")(data)
    assert ieq("ABC")(data)
    assert eq("b")("b")
    assert eq(10)([1, 2, 10])
    assert eq(10.0)([1, 2, 10.0])
    assert eq(10)(10)
    assert eq(10.1)(10.1)


def test_less_than():
    data = ["abc", "abrd", "ed"]
    assert lt("abd")(data)
    assert ilt("ABD")(data)
    assert lt("b")("a")
    assert lt(10)([1, 2, 10])
    assert lt(10)(9)
    assert lt(10.2)([1, 2, 10.1])
    assert lt(10.2)(9.3)


def test_less_than_equals():
    data = ["abc", "abrd", "ed"]
    assert le("abd")(data)
    assert ile("ABD")(data)
    assert le("abc")(data)
    assert le("b")("b")
    assert le("b")("a")
    assert le(1)([1, 2, 10])
    assert le(10)(9.9)
    assert le(10.0)(10)
    assert le(10.1)([1, 2, 10.01])
    assert le(2.3)([1, 2.3, 10.1])
    assert le(10.2)(9.3)
    assert le(9.3)(9.3)


def test_greater_than():
    data = ["abc", "abrd", "ed"]
    assert gt("abb")(data)
    assert igt("ABB")(data)
    assert gt("b")("c")
    assert gt(10)([1, 2, 11])
    assert gt(10)(11)
    assert gt(10.4)([1, 2, 11.5])
    assert gt(10.9)(11.02)


def test_greater_than_equals():
    data = ["abc", "abrd", "ed"]
    assert ge("abb")(data)
    assert ige("ABB")(data)
    assert ge("abc")(data)
    assert ge("b")("c")
    assert ge("c")("c")
    assert ge(10)([1, 2, 11])
    assert ge(11)([1, 2, 11])
    assert ge(10)(11)
    assert ge(11)(11)
    assert ge(10.4)([1, 2, 11.5])
    assert ge(11.5)([1, 2, 11.5])
    assert ge(10.9)(11.02)
    assert ge(11.02)(11.02)


def test_simple_queries():
    httpd1 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_1, path='/etc/httpd/conf/httpd.conf'))
    result = HttpdConfTree([httpd1])
    assert result["EnableSendfile"][first].value
    assert len(result["VirtualHost"]) == 1
    assert len(result["VirtualHost"]["Directory"]) == 1
    assert len(result["VirtualHost"]["IfModule"]) == 4
    assert len(result["VirtualHost"]["IfModule"]["RewriteEngine"]) == 2
    assert len(result["VirtualHost"]["IfModule", "mod_rewrite.c"]) == 2


def test_complex_queries():
    httpd1 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_1, path='/etc/httpd/conf/httpd.conf'))
    httpd2 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_3, path='/etc/httpd/conf.d/00-a.conf'))
    httpd3 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_4, path='/etc/httpd/conf.d/01-b.conf'))
    result = HttpdConfTree([httpd1, httpd2, httpd3])
    assert len(result.select("VirtualHost")) == 3
    assert len(result.select(("VirtualHost", "128.39.140.28"))) == 2
    assert len(result.select(("VirtualHost", "128.39.140.30"))) == 1
    assert len(result.select(("VirtualHost", startswith("128.39.140")))) == 3
    assert len(result.select(("VirtualHost", ~startswith("128.39.140")))) == 0
    assert len(result.select(("VirtualHost", endswith("140.30")))) == 1
    assert len(result.select(("VirtualHost", ~endswith("140.30")))) == 2
    assert len(result.select((startswith("Virtual"), ~endswith("140.30")))) == 2
    assert len(result.select("FilesMatch", deep=True, roots=False)) == 3

    # find all IfModule !php5_module stanzas regardless of location
    assert len(result.select(("IfModule", "!php5_module"), deep=True, roots=False)) == 6

    # find all IfModule !php5_module stanzas immediately beneath a VirtualHost
    assert len(result.select("VirtualHost", ("IfModule", "!php5_module"), roots=False)) == 3

    # find all IfModule !php4_module stanzas anywhere beneath a top level VirtualHost
    res = result.select("VirtualHost").select(("IfModule", "!php4_module"), deep=True, roots=False)
    assert len(res) == 3

    assert len(result.select(("VirtualHost", ~is_private))) == 3

    res = result.select(("VirtualHost", in_network("128.39.0.0/16")))
    assert len(res) == 3

    res = result.select(("VirtualHost", ~is_private & in_network("128.39.0.0/16")))
    assert len(res) == 3


def test_directives_and_sections():
    httpd1 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_1, path='/etc/httpd/conf/httpd.conf'))
    httpd2 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_3, path='/etc/httpd/conf.d/00-a.conf'))
    httpd3 = _HttpdConf(context_wrap(HTTPD_CONF_NEST_4, path='/etc/httpd/conf.d/01-b.conf'))
    result = HttpdConfTree([httpd1, httpd2, httpd3])
    assert len(result.directives) == 3
    assert len(result.sections) == 7
    assert len(result.find_all(startswith("Dir")).directives) == 1
    assert len(result.find_all(startswith("Dir")).sections) == 1



"""
PCSStatus - command ``pcs status``
==================================

This module provides the classs ``PCSStatus`` which processes
``/usr/sbin/pcs status`` command output. Typical output of the ``pcs status``
command looks like::

        Cluster name: mycluster
        Last updated: Thu Dec  1 02:33:50 2016		Last change: Wed Aug  3 03:47:11 2016 by root via cibadmin on nodea.example.com
        Stack: corosync
        Current DC: nodea.example.com (version 1.1.13-10.el7-44eb2dd) - partition WITHOUT quorum
        3 nodes and 3 resources configured

        Online: [ nodea.example.com ]
        OFFLINE: [ nodeb.example.com nodec.example.com ]

        Full list of resources:
         myfence	(stonith:fence_xvm):	Stopped
         Resource Group: myweb
             webVIP	(ocf::heartbeat:IPaddr2):	Stopped
             webserver	(ocf::heartbeat:apache):	Stopped
        PCSD Status:
          nodea.example.com: Online
          nodeb.example.com: Offline
          nodec.example.com: Offline
        Daemon Status:
          corosync: active/enabled
          pacemaker: active/enabled
          pcsd: active/enabled

The class ``PCSStatus`` has one attribute ``data`` which is a dict containing
the whole the output and one attribute ``nodes`` is a list containing all
node names that from ``PCSD Status`` section.


Examples:

    >>> pcsstatus_content = '''
    ... Cluster name: openstack
    ... Last updated: Fri Oct 14 15:45:32 2016
    ... Last change: Thu Oct 13 20:02:27 2016
    ... Stack: corosync
    ... Current DC: myhost15 (1) - partition with quorum
    ... Version: 1.1.12-a14efad
    ... 3 Nodes configured
    ... 143 Resources configured
    ... online: [ myhost15 myhost16 myhost17 ]
    ... Full list of resources:
    ... stonith-ipmilan-10.24.221.172	(stonith:fence_ipmilan):	Started myhost15
    ... stonith-ipmilan-10.24.221.171	(stonith:fence_ipmilan):	Started myhost16
    ... stonith-ipmilan-10.24.221.173	(stonith:fence_ipmilan):	Started myhost15
    ... PCSD Status:
    ...     myhost15: Online
    ...     myhost17: Online
    ...     myhost16: Online
    ... Daemon Status:
    ...    corosync: active/enabled
    ...    pacemaker: active/enabled
    ...    pcsd: active/enabled
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> shared = {PCSStatus: PCSStatus(context_wrap(pcsstatus_content))}
    >>> pcsstatus_info = shared[PCSStatus]
    >>> pcsstatus_info.get("Cluster name")
    'openstack'
    >>> pcsstatus_info.get("Stack")
    'corosync'
    >>> pcsstatus_info.get("Nodes configured")
    '3'
    >>> pcsstatus_info.get("Resources configured")
    '143'
    >>> pcsstatus_info.nodes
    ['myhost15', 'myhost17', 'myhost16']
    >>> len(pcsstatus_info.get("Full list of resources"))
    3
"""
from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.pcs_status)
class PCSStatus(CommandParser):
    """Class to process the output of ``pcs status`` command.

    Attributes:
        nodes (list): A list containing all node names according "PCSD Status" section
        data (dict): A dict containing key, value for each line of the output in
            the form::

             {
                "Resources configured": "143",
                "PCSD Status": [
                    "myhost15: Online",
                    "myhost17: Online",
                    "myhost16: Online"
                ],
                "Current DC": "myhost15 (1) - partition with quorum",
                "Full list of resources": [
                    "stonith-ipmilan-10.24.221.172\t(stonith:fence_ipmilan):\tStarted myhost15",
                    "stonith-ipmilan-10.24.221.171\t(stonith:fence_ipmilan):\tStarted myhost16",
                    "stonith-ipmilan-10.24.221.173\t(stonith:fence_ipmilan):\tStarted myhost15",
                ],
                "Daemon Status": [
                    "corosync: active/enabled",
                    "pacemaker: active/enabled",
                    "pcsd: active/enabled"
                ],
                "Nodes configured": "3",
                "Online": "[ myhost15 myhost16 myhost17 ]",
                "Cluster name": "openstack",
                "Stack": "corosync"
             }
    """

    def parse_content(self, content):
        self.nodes = []
        self.data = {}
        self.bad_nodes = []
        # according this file  https://github.com/ClusterLabs/pacemaker/blob/d078ca4a9ac72fc96073a215da2eed48939536f5/tools/crm_mon.c
        key_oneline = (
            "Cluster name", "Stack", "Current DC", "Online", "RemoteOnline", "OFFLINE", "RemoteOFFLINE", "GuestOnline")
        key_nextline = ("Full list of resources", "Failed Actions", "PCSD Status", "Daemon Status")
        key_nextline_start = 0
        multiple_lines = []
        for line in content:
            if line.startswith("WARNING:"):
                if "WARNING" in self.data:
                    self.data["WARNING"].append(line.strip())
                else:
                    self.data["WARNING"] = [line.strip()]
                continue
            if (line.startswith("RemoteNode") or line.startswith("GuestNode") or line.startswith("Node")):
                linesplit = line.split()
                bad_node = {}
                bad_node['name'] = linesplit[1][0:-1]
                bad_node['status'] = line.split(':')[1][1:]
                self.bad_nodes.append(bad_node)
            if line.startswith(key_nextline):
                key_nextline_start = 1
                multiple_lines = []
                self.data[line.split(":")[0]] = multiple_lines
                continue
            if key_nextline_start == 0:
                linesplit = line.split()
                if line.startswith(key_oneline):
                    key, value = line.split(":", 1)
                    self.data[key.strip()] = value.strip()
                elif "Nodes configured" in line:
                    self.data["Nodes configured"] = linesplit[0]
                elif "Resources configured" in line:
                    self.data["Resources configured"] = linesplit[0]
                elif "resources configured" in line:
                    if "and" in line:  # 3 nodes and 3 resources configured
                        self.data['Nodes configured'] = linesplit[0]
                        self.data["Resources configured"] = linesplit[3]
            else:  # key_nextline_start > 0
                if line.strip():
                    multiple_lines.append(line.strip())
        pcsd_status = self.data.get("PCSD Status")
        if pcsd_status:
            # According "PCSD Status" to get node list
            for line in pcsd_status:
                self.nodes.append(line.split(":")[0].strip())

    def get(self, i_key):
        """str/list: Returns the data associated with ``i_key`` or ``None`` if ``i_key`` is not present"""
        return self.data.get(i_key)



"""
Hostname
========

Combiner for ``hostname`` information. It uses the results of the
``Hostname`` parser, ``Facter`` and the ``SystemID`` parser to get the fqdn,
hostname and domain information.

Examples:
    >>> hn = shared[Hostname]
    >>> hn.fqdn
    rhel7.redhat.com
    >>> hn.hostname
    rhel7
    >>> hn.domain
    redhat.com
    >>> hn
    Hostname(fqdn='rhel7.redhat.com', hostname='rhel7', domain='redhat.com')

"""

from collections import namedtuple
from insights.core.plugins import combiner
from insights.core.serde import deserializer, serializer
from insights.parsers.hostname import Hostname as hname
from insights.parsers.facter import Facter as facter
from insights.parsers.systemid import SystemID as systemid

Hostname = namedtuple("Hostname", field_names=["fqdn", "hostname", "domain"])
"""namedtuple: Type for storing the hostname information."""


@serializer(Hostname)
def serialize(obj, root=None):
    return {"fqdn": obj.fqdn, "hostname": obj.hostname, "domain": obj.domain}


@deserializer(Hostname)
def deserialize(_type, data, root=None):
    return Hostname(**data)


@combiner([hname, facter, systemid])
def hostname(hn, ft, si):
    """Check hostname, facter and systemid to get the fqdn, hostname and domain.

    Prefer hostname to facter and systemid.

    Returns:
        insights.combiners.hostname.Hostname: A named tuple with `fqdn`,
        `hostname` and `domain` components.

    Raises:
        Exception: If no hostname can be found in any of the three parsers.
    """

    if not hn or not hn.fqdn:
        hn = ft

    if hn and hn.fqdn:
        fqdn = hn.fqdn
        hostname = hn.hostname if hn.hostname else fqdn.split(".")[0]
        domain = hn.domain if hn.domain else ".".join(fqdn.split(".")[1:])
        return Hostname(fqdn, hostname, domain)
    else:
        fqdn = si.get("profile_name") if si else None
        if fqdn:
            hostname = fqdn.split(".")[0]
            domain = ".".join(fqdn.split(".")[1:])
            return Hostname(fqdn, hostname, domain)

    raise Exception("Unable to get hostname.")



"""
Yum - Commands
==============
Parsers for ``yum`` commands.

This module contains the classes that parse the output of the commands
`yum -C repolist`.

YumRepoList - command ``yum -C repolist``
-----------------------------------------
"""
from insights import parser, CommandParser
from insights.parsers import SkipException
from insights.specs import Specs

eus = [
    '5.0.z',
    '5.1.z',
    '5.2.z',
    '5.3.ll',
    '5.3.z',
    '5.4.z',
    '5.6.ll',
    '5.6.z',
    '5.9.ll',
    '5.9.z'
    '6.0.z',
    '6.1.z',
    '6.2.aus',
    '6.2.z',
    '6.3.z',
    '6.4.aus',
    '6.4.z',
    '6.5.aus',
    '6.5.z',
    '6.6.aus',
    '6.6.z',
    '6.7.z'
]


@parser(Specs.yum_repolist)
class YumRepoList(CommandParser):
    """
    Class for parsing the output of `yum -C repolist` command.

    Typical output of the command is::

        Loaded plugins: langpacks, product-id, search-disabled-repos, subscription-manager
        repo id                                             repo name                                                                                                    status
        rhel-7-server-e4s-rpms/x86_64                       Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)                                 12,250
        rhel-ha-for-rhel-7-server-e4s-rpms/x86_64           Red Hat Enterprise Linux High Availability (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)         272
        rhel-ha-for-rhel-7-server-rpms/x86_64               Red Hat Enterprise Linux High Availability (for RHEL 7 Server) (RPMs)                                           225
        rhel-sap-hana-for-rhel-7-server-e4s-rpms/x86_64     RHEL for SAP HANA (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)                                   21
        repolist: 12,768

    Examples:
        >>> len(repolist)
        4
        >>> 'rhel-7-server-e4s-rpms/x86_64' in repolist.repos
        True
        >>> 'rhel-7-server-e4s-rpms' in repolist.repos
        False
        >>> 'rhel-7-server-e4s-rpms' in repolist.rhel_repos
        True
        >>> repolist['rhel-7-server-e4s-rpms/x86_64']['name']
        'Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)'
        >>> repolist[0]['name']
        'Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)'

    Attributes:
        data (list): list of repos wrapped in dictionaries
        repos (dict): dict of all listed repos where the key is the full repo-id

    """
    def parse_content(self, content):
        self.data = []
        self.repos = {}
        found_start = False
        for line in content:
            if line.startswith("repolist:"):
                break
            if found_start:
                _id, right = line.split(None, 1)
                try:
                    name, status = right.rsplit(None, 1)
                except ValueError:
                    raise SkipException("Incorrect line: '{0}'".format(line))
                self.data.append({
                    "id": _id.strip(),
                    "name": name.strip(),
                    "status": status.strip()})
            if not found_start:
                found_start = line.startswith("repo id")
        if not self.data:
            raise SkipException('No repolist.')
        self.repos = dict((d['id'], d) for d in self.data)

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        if isinstance(idx, int):
            return self.data[idx]
        elif isinstance(idx, str):
            return self.repos[idx]

    def __contains__(self, _id):
        return _id in self.repos

    @property
    def eus(self):
        '''list of the EUS part of each repo'''
        euses = []
        for repo in [r["id"] for r in self.data]:
            if repo.startswith("rhel-") and "server-" in repo:
                _, eus_version = repo.split("server-", 1)
                if eus_version in eus:
                    euses.append(eus_version)
        return euses

    @property
    def rhel_repos(self):
        '''list of RHEL repos/Repo IDs'''
        return [i.split('/')[0].lstrip('!')
                for i in self.repos
                if i.startswith(('!rhel', 'rhel'))]



"""
This module models ini configuration as a tree.

A DSL is provided to query the tree through a select function or brackets [].
The brackets allow a more conventional lookup feel but aren't quite as powerful
as using select directly.
"""
import re

from insights.configtree import Directive, DocParser, LineGetter, Section
from insights.configtree import eq, first, last, typed


class ConfigParser(DocParser):
    def parse_directive(self, lg):
        line = next(lg)
        parts = [p.strip() for p in re.split(":|=", line, 1)]

        name = parts[0].strip()
        attrs = parts[1:]

        try:
            # handle continuations
            spaces = " " * (line.expandtabs().find(name) + 1)
            while lg.peek().expandtabs().startswith(spaces):
                attrs.append(next(lg).lstrip())
        except:
            pass

        if attrs:
            attrs = ["\n".join(attrs)] if len(attrs) > 1 else [typed(attrs[0])]
        return Directive(name=name, attrs=attrs, ctx=self.ctx)

    def parse_section_body(self, lg):
        body = []
        try:
            while not lg.peek().lstrip().startswith("["):
                body.append(self.parse_statement(lg))
        except StopIteration:
            pass
        return body

    def parse_section(self, lg):
        name = next(lg).strip("[] ")
        body = self.parse_section_body(lg)
        return Section(name=name, children=body, ctx=self.ctx)

    def parse_statement(self, lg):
        line = lg.peek()
        pos = lg.pos
        if line.lstrip().startswith("["):
            el = self.parse_section(lg)
        else:
            el = self.parse_directive(lg)
        el.pos = pos
        return el


def squash(doc):
    seen = set()
    add = seen.add
    names = [c.name for c in doc if not (c.name in seen or add(c.name))]
    results = [doc.select(n, one=last, roots=False) for n in names]
    doc.children = results
    for c in doc:
        squash(c)


def set_defaults(cfg):
    if "DEFAULT" not in cfg:
        return

    defaults = cfg.select("DEFAULT", one=first)
    for c in cfg[~eq("DEFAULT")]:
        for d in defaults:
            if d.name not in c:
                c.children.append(d)

    cfg.children = list(cfg[~eq("DEFAULT")])


def parse_doc(f, ctx=None, overwrite=False):
    """ Accepts an open file or a list of lines. """
    lg = LineGetter(f, comment_marker=("#", ";"), strip=False)
    cfg = ConfigParser(ctx).parse_doc(lg)
    set_defaults(cfg)
    if overwrite:
        squash(cfg)
    return cfg



import doctest
import pytest
from insights.parsers import ParseException
from insights.parsers.bond import Bond
from insights.parsers import bond
from insights.tests import context_wrap

CONTEXT_PATH = "proc/net/bonding/bond0"

BONDINFO_1 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: load balancing (round-robin)
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: eno1
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f8
Slave queue ID: 0

Slave Interface: eno2
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f9
Slave queue ID: 0
""".strip()

BONDINFO_MODE_4 = """
Ethernet Channel Bonding Driver: v3.2.4 (January 28, 2008)

Bonding Mode: IEEE 802.3ad Dynamic link aggregation
Transmit Hash Policy: layer2 (0)
MII Status: up
MII Polling Interval (ms): 500
Up Delay (ms): 0
Down Delay (ms): 0

802.3ad info
LACP rate: slow
Active Aggregator Info:
        Aggregator ID: 3
        Number of ports: 1
        Actor Key: 17
        Partner Key: 1
        Partner Mac Address: 00:00:00:00:00:00

Slave Interface: eth1
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 00:16:35:5e:42:fc
Aggregator ID: 3

Slave Interface: eth2
MII Status: up
Speed: 1000 Mbps
Duplex: full
Speed
Link Failure Count: 0
Permanent HW addr: 00:16:35:5e:02:7e
Aggregator ID: 2
""".strip()

BONDINFO_MODE_2 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: load balancing (xor)
Transmit Hash Policy: layer2+3 (2)
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: eno1
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f8
Slave queue ID: 0

Slave Interface: eno2
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f9
Slave queue ID: 0
""".strip()


BONDINFO_CORRUPT = """
Link Failure Count: 0
Permanent HW addr: 00:16:35:5e:02:7e
Aggregator ID:
""".strip()

BONDINFO_UNKNOWN_BOND_MODE = """
Bonding Mode: reverse proximity hash combination mode
""".strip()


BONDINFO_MODE_5 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: fault-tolerance (active-backup)
Primary Slave: None
Currently Active Slave: enp17s0f0
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: enp17s0f0
MII Status: up
Speed: 10000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 00:1f:f3:af:d3:f0
Slave queue ID: 0

Slave Interface: enp17s0f1
MII Status: up
Speed: 10000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 00:1f:f3:af:d3:f1
Slave queue ID: 0
""".strip()


BONDINFO_MODE_6 = BONDINFO_MODE_5.replace("Currently Active Slave: enp17s0f0", "")


def test_netstat_doc_examples():
    env = {
        'bond_info': Bond(context_wrap(BONDINFO_MODE_4)),
    }
    failed, total = doctest.testmod(bond, globs=env)
    assert failed == 0


def test_bond_class():
    bond_obj = Bond(context_wrap(BONDINFO_1, CONTEXT_PATH))
    assert bond_obj.file_name == 'bond0'
    assert not bond_obj.partner_mac_address
    assert bond_obj.bond_mode == '0'
    assert bond_obj.slave_interface == ['eno1', 'eno2']

    bond_obj = Bond(context_wrap(BONDINFO_MODE_4, CONTEXT_PATH))
    assert bond_obj.bond_mode == '4'
    assert bond_obj.partner_mac_address == "00:00:00:00:00:00"
    assert bond_obj.aggregator_id == ['3', '3', '2']
    assert bond_obj.xmit_hash_policy == "layer2"
    assert bond_obj.active_slave is None

    bond_obj = Bond(context_wrap(BONDINFO_CORRUPT, CONTEXT_PATH))
    assert bond_obj.bond_mode is None
    assert bond_obj.slave_interface == []
    assert not bond_obj.xmit_hash_policy

    bond_obj = Bond(context_wrap(BONDINFO_MODE_2, CONTEXT_PATH))
    assert bond_obj.xmit_hash_policy == "layer2+3"

    bond_obj = Bond(context_wrap(BONDINFO_MODE_5, CONTEXT_PATH))
    assert bond_obj.bond_mode == '1'
    assert bond_obj.active_slave == "enp17s0f0"

    bond_obj_2 = Bond(context_wrap(BONDINFO_MODE_6, CONTEXT_PATH))
    assert bond_obj_2.bond_mode == '1'
    assert bond_obj_2.active_slave is None

    bond_obj_3 = Bond(context_wrap(BONDINFO_1, CONTEXT_PATH))
    assert bond_obj_3.file_name == 'bond0'
    assert bond_obj_3.slave_interface == ['eno1', 'eno2']
    assert bond_obj_3.slave_duplex == ['full', 'full']
    assert bond_obj_3.slave_speed == ['1000 Mbps', '1000 Mbps']
    assert bond_obj_3.slave_link_failure_count == ['0', '0']
    assert bond_obj_3.mii_status == ['up', 'up', 'up']

    with pytest.raises(ParseException) as exc:
        bond_obj = Bond(context_wrap(BONDINFO_UNKNOWN_BOND_MODE, CONTEXT_PATH))
        assert not bond_obj.bond_mode
    assert 'Unrecognised bonding mode' in str(exc)



"""
Ethtool parsers
===============

Classes to parse ``ethtool`` command information.

The interface information for a machine is stored as lists.  Each interface
is accessed by iterating through the shared parser list.

The interface classes all provide the following properties:

* ``iface`` and ``ifname``: the interface name (derived from the output file).
* ``data``: the data for that interface

Parsers provided by this module include:

CoalescingInfo - command ``/sbin/ethtool -c {interface}``
---------------------------------------------------------

Driver - command ``/sbin/ethtool -i {interface}``
-------------------------------------------------

Ethtool - command ``/sbin/ethtool {interface}``
-----------------------------------------------

Features - command ``/sbin/ethtool -k {interface}``
---------------------------------------------------

Pause - command ``/sbin/ethtool -a {interface}``
------------------------------------------------

Ring - command ``/sbin/ethtool -g {interface}``
-----------------------------------------------

Statistics - command ``/sbin/ethtool -S {interface}``
-----------------------------------------------------

"""

import os
import re
from collections import namedtuple
from ..parsers import ParseException
from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs


def extract_iface_name_from_path(path, name):
    """
    Extract the 'real' interface name from the path name.  Basically this
    puts the '@' back in the name in place of the underscore, where the name
    contains a '.' or contains 'macvtap' or 'macvlan'.

    Examples:

    +------------------+-----------------+
    | real name        | path name       |
    +==================+=================+
    | bond0.104\@bond0 | bond0.104_bond0 |
    +------------------+-----------------+
    | __tmp1111        | __tmp1111       |
    +------------------+-----------------+
    | macvtap\@bond0   | macvlan_bond0   |
    +------------------+-----------------+
    | prod_bond        | prod_bond       |
    +------------------+-----------------+
    """
    if name in path:
        ifname = os.path.basename(path).split("_", 2)[-1].strip()
        if "." in ifname or "macvtap" in ifname or "macvlan" in ifname:
            ifname = ifname.replace("_", "@")
        return ifname


def extract_iface_name_from_content(content):
    """
    Extract the interface name from the third item in the content, delimited
    by spaces, up to its second-last character.  For example, this transmutes
    ``Features for bond0:`` to ``bond0``.
    """
    return content.split(" ", 3)[-1][:-1]


@parser(Specs.ethtool_i)
class Driver(CommandParser):
    """
    Parse information for the ``ethtool -i`` command.

    All the ``ethtool -i`` outputs are stored as a list, in no particular
    order.

    Each driver is stored as a dictionary in the ``data`` property.  If the
    key starts with 'supports', then the value is a boolean test of whether
    the string is 'yes'.  If the value is not given on the string (e.g.
    'bus-info:'), the value is set to None.

    All data is also set as attributes of the object with the attribute name
    being the key name with hyphens replaced with underscores.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.
        driver (str): The driver providing the interface.
        version (str): The version of the interface driver.
        firmware_version (str): The firmware version of the interface.
        supports_statistics (bool): Does the interface support statistics
            gathering?
        supports_test (bool): Does the interface support internal self-tests?
        supports_eeprom_access (bool): Does the interface support access to
            the EEPROM?
        supports_register_dump (bool): Does the interface support dumping the
            internal registers?
        supports_priv_flags (bool): Does the interface support use of
            privileged flags?

    Sample input for ``/sbin/ethtool -i bond0``::

        driver: bonding
        version: 3.6.0
        firmware-version: 2
        bus-info:
        supports-statistics: no
        supports-test: no
        supports-eeprom-access: no
        supports-register-dump: no
        supports-priv-flags: no

    Examples::
        >>> len(interfaces) # All interfaces in a list
        1
        >>> type(interfaces[0])
        <class 'insights.parsers.ethtool.Driver'>
        >>> bond0 = interfaces[0] # Would normally iterate through interfaces
        >>> bond0.iface
        'bond0'
        >>> bond0.ifname
        'bond0'
        >>> bond0.data['driver'] # Old-style access
        'bonding'
        >>> bond0.driver # New-style access
        'bonding'
        >>> hasattr(bond0, 'bus_info')
        True
        >>> bond0.bus_info is None
        True
        >>> bond0.supports_statistics
        False
    """

    @property
    def ifname(self):
        """(str): the interface name"""
        return self.iface

    def parse_content(self, content):
        self.iface = extract_iface_name_from_path(self.file_path, "ethtool_-i_")
        self.data = {}
        self.driver = None
        self.version = None
        self.firmware_version = None
        self.bus_info = None
        self.supports_statistics = None
        self.supports_test = None
        self.supports_eeprom_access = None
        self.supports_register_dump = None
        self.supports_priv_flags = None

        for line in content:
            if ":" in line:
                key, value = [s.strip() for s in line.split(":", 1)]
                value = value if value else None
                if key.startswith("supports"):
                    value = value == "yes"
                self.data[key] = value
                setattr(self, key.replace("-", "_"), value)


@parser(Specs.ethtool_k)
class Features(LegacyItemAccess, CommandParser):
    """
    Parse information for the ``ethtool -k`` command.

    Features are stored as a flat set of key: value pairs, with no hierarchy
    that the indentation of the input might imply.  This means, that the
    output below will provide data for 'tx-checksumming' and
    'tx-checksum-ipv4'.

    Each key stores a two-key dictionary:

    * 'on' (boolean) - whether the value (before any '[fixed]') is 'on'.
    * 'fixed' (boolean) - whether the value contains 'fixed'.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.

    Sample input for ``/sbin/ethtool -k bond0``::

        Features for bond0:
        rx-checksumming: off [fixed]
        tx-checksumming: on
            tx-checksum-ipv4: off [fixed]
            tx-checksum-unneeded: on [fixed]
            tx-checksum-ip-generic: off [fixed]
            tx-checksum-ipv6: off [fixed]
            tx-checksum-fcoe-crc: off [fixed]
            tx-checksum-sctp: off [fixed]
        scatter-gather: on
            tx-scatter-gather: on [fixed]
            tx-scatter-gather-fraglist: off [fixed]
        tcp-segmentation-offload: on
            tx-tcp-segmentation: on [fixed]
            tx-tcp-ecn-segmentation: on [fixed]
            tx-tcp6-segmentation: on [fixed]
        udp-fragmentation-offload: off [fixed]
        generic-segmentation-offload: off [requested on]
        generic-receive-offload: on
        large-receive-offload: on
        rx-vlan-offload: on
        tx-vlan-offload: on
        ntuple-filters: off
        receive-hashing: off
        highdma: on [fixed]
        rx-vlan-filter: on [fixed]
        vlan-challenged: off [fixed]
        tx-lockless: on [fixed]
        netns-local: off [fixed]
        tx-gso-robust: off [fixed]
        tx-fcoe-segmentation: off [fixed]
        tx-gre-segmentation: on [fixed]
        tx-udp_tnl-segmentation: on [fixed]
        fcoe-mtu: off [fixed]
        loopback: off [fixed]

    Examples:

        >>> len(features) # All interfaces in a list
        1
        >>> type(features[0])
        <class 'insights.parsers.ethtool.Features'>
        >>> bond0 = features[0] # Would normally iterate through interfaces
        >>> bond0.iface
        'bond0'
        >>> bond0.ifname
        'bond0'
        >>> bond0.data['rx-vlan-offload']['on'] # Traditional access
        True
        >>> bond0.data['rx-vlan-offload']['fixed']
        False
        >>> bond0.data['tx-checksum-sctp']['on']
        False
        >>> bond0.data['tx-checksum-sctp']['fixed']
        True
        >>> bond0.is_on('ntuple-filters')
        False
        >>> bond0.is_on('large-receive-offload')
        True
        >>> bond0.is_fixed('receive-hashing')
        False
        >>> bond0.is_fixed('fcoe-mtu')
        True
    """

    @property
    def ifname(self):
        """(str): the interface name"""
        return self.iface

    def is_on(self, feature):
        """(bool): Does this feature exist and is it on?"""
        return self.get(feature, {}).get('on', False)

    def is_fixed(self, feature):
        """(bool): Does this feature exist and is it fixed?"""
        return self.get(feature, {}).get('fixed', False)

    def parse_content(self, content):
        self.data = {}
        self.iface = extract_iface_name_from_path(self.file_path, "ethtool_-k_")
        # Handle e.g. '/sbin/ethtool: file not found'
        if "ethtool" in content[0]:
            return
        # Handle 'Cannot get feature settings: Operation not supported'
        if "Cannot get" in content[0]:
            return

        # Need to strip header line that's only on -k
        for line in content[1:]:
            if ":" in line:
                key, value = [s.strip() for s in line.strip().split(":", 1)]
                fixed = "fixed" in value
                if fixed:
                    value = value.split()[0].strip()
                self.data[key.strip()] = {
                    "on": value == "on",
                    "fixed": fixed
                }


@parser(Specs.ethtool_a)
class Pause(CommandParser):
    """
    Parse information for the ``ethtool -a`` command.

    Each parameter in the input is stored as a key in a dictionary, with the
    value being whether the found string equals 'on'.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.
        autonegotiate (bool): Is autonegotiate present and set to 'on'?
        rx (bool): Is receive pausing present and set to 'on'?
        tx (bool): Is transmit pausing present and set to 'on'?
        rx_negotiated (bool): Is receive pause autonegotiate present and set to 'on'?
        tx_negotiated (bool): Is transmit pause autonegotiate present and set to 'on'?

    Sample input from ``/sbin/ethtool -a 0``::

        Pause parameters for eth0:
        Autonegotiate:  on
        RX:             on
        TX:             on
        RX negotiated:  off
        TX negotiated:  off

    Examples:
        >>> len(pause) # All interfaces in a list
        1
        >>> type(pause[0])
        <class 'insights.parsers.ethtool.Pause'>
        >>> eth0 = pause[0] # Would normally iterate through interfaces
        >>> eth0.iface
        'eth0'
        >>> eth0.ifname
        'eth0'
        >>> eth0.data['RX'] # Old-style accessor
        True
        >>> eth0.autonegotiate # New-style accessor
        True
        >>> eth0.rx_negotiated
        False
    """

    @property
    def ifname(self):
        return self.iface

    @property
    def autonegotiate(self):
        return self.data.get("Autonegotiate", False)

    @property
    def rx(self):
        return self.data.get("RX", False)

    @property
    def tx(self):
        return self.data.get("TX", False)

    @property
    def rx_negotiated(self):
        return self.data.get("RX Autonegotiate", False)

    @property
    def tx_negotiated(self):
        return self.data.get("TX Autonegotiate", False)

    def parse_content(self, content):
        """
        Return ethtool -a result as a dict.

        If ethtool -a outputs an error or could not get the pause state for
        the device, the "iface" property will be set but the data dictionary
        will be blank and all properties will return False.
        """
        self.data = {}
        self.iface = extract_iface_name_from_path(self.file_path, "ethtool_-a_")
        # Handle e.g. '/sbin/ethtool: file not found'
        if "ethtool" in content[0]:
            return
        # Handle 'Cannot get device pause settings: Operation not supported'
        if "Cannot get" in content[0]:
            return

        for line in content[1:]:
            if line.strip():
                (key, value) = [s.strip() for s in line.split(":", 1)]
                self.data[key] = (value == "on")
                # Can't use key if it has a space in it, and we provide these
                # as properties anyway.
                # setattr(self, key, value == "on")


@parser(Specs.ethtool_c)
class CoalescingInfo(CommandParser):
    """
    Parse information for the ``ethtool -c`` command.

    The parsing is fairly similar to other ``ethtool`` parsers - the
    interface name is available as the ``ifname`` and ``iface`` properties,
    and the data about the coalescing information is stored in the ``data``
    property as a dictionary.  The one difference is the 'Adaptive RX' data,
    which is stored as two keys - 'adaptive-rx' and 'adaptive-tx', for RX
    and TX respectively.  Both these return a boolean for whether the
    respective state equals 'on'.

    Otherwise, all values are made available as keys in the ``data``
    dictionary, and as properties with the hyphen transmuted to an underscore
    - e.g. ``obj.data['tx-usecs']`` is available as ``obj.tx_usecs``.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.

    Sample input for ``/sbin/ethtool -c eth0``::

        Coalesce parameters for eth0:
        Adaptive RX: off  TX: off
        stats-block-usecs: 0
        sample-interval: 0
        pkt-rate-low: 0
        pkt-rate-high: 0

        rx-usecs: 20
        rx-frames: 5
        rx-usecs-irq: 0
        rx-frames-irq: 5

        tx-usecs: 72
        tx-frames: 53
        tx-usecs-irq: 0
        tx-frames-irq: 5

        rx-usecs-low: 0
        rx-frame-low: 0
        tx-usecs-low: 0
        tx-frame-low: 0

        rx-usecs-high: 0
        rx-frame-high: 0
        tx-usecs-high: 0
        tx-frame-high: 0

    Examples:
        >>> len(coalesce) # All interfaces in a list
        1
        >>> type(coalesce[0])
        <class 'insights.parsers.ethtool.CoalescingInfo'>
        >>> eth0 = coalesce[0] # Would normally iterate through interfaces
        >>> eth0.iface
        'eth0'
        >>> eth0.ifname
        'eth0'
        >>> eth0.data['adaptive-rx'] # Old-style accessor
        False
        >>> eth0.adaptive_rx # New-style accessor
        False
        >>> eth0.rx_usecs # Note integer value
        20
    """

    @property
    def ifname(self):
        """(str): the interface name"""
        return self.iface

    def parse_content(self, content):
        """
        Parse the output of ``ethtool -c`` into a dictionary.

        If ethtool -c outputs an error or could not get the pause state for
        the device, the "iface" property will be set but the data dictionary
        will be blank.
        """
        content = list(content)
        self.data = {}
        self.iface = extract_iface_name_from_path(self.file_path, "ethtool_-c_")

        if "ethtool" in content[0]:
            return

        if "Cannot get" in content[0]:
            # cannot got pause param in ethtool
            self.iface = extract_iface_name_from_content(content[1])
            return

        self.iface = extract_iface_name_from_content(content[0])

        if len(content) <= 1:
            raise ParseException("Command output missing value data")

        second_line_content = content[1].split(" ")
        self.data["adaptive-rx"] = (second_line_content[2] == "on")
        self.adaptive_rx = (second_line_content[2] == "on")
        self.data["adaptive-tx"] = (second_line_content[5] == "on")
        self.adaptive_tx = (second_line_content[5] == "on")

        for line in content[2:]:
            if line.strip():
                (key, value) = [s.strip() for s in line.split(":", 1)]
                value = int(value)
                self.data[key] = value
                setattr(self, key.replace("-", "_"), value)


@parser(Specs.ethtool_g)
class Ring(CommandParser):
    """
    Parse information for the ``ethtool -g`` command.

    In addition to the standard ``iface`` and ``ifname`` parameters, as well
    as being available in the ``data`` property, the interface statistics
    are accessed using two parameters: ``max`` and ``current``.
    Within each the interface settings are available as four parameters -
    ``rx``, ``rx_mini``, ``rx_jumbo`` and ``tx``.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.
        max (dict): Dictonary of maximum ring buffer settings.
        current (dict): Dictionary of current ring buffer settings.

    Sample input for ``/sbin/ethtool -g eth0``::

        Ring parameters for eth0:
        Pre-set maximums:
        RX:             2047
        RX Mini:        0
        RX Jumbo:       0
        TX:             511
        Current hardware settings:
        RX:             200
        RX Mini:        0
        RX Jumbo:       0
        TX:             511


    Examples:
        >>> len(ring) # All interfaces in a list
        1
        >>> type(ring[0])
        <class 'insights.parsers.ethtool.Ring'>
        >>> eth0 = ring[0] # Would normally iterate through interfaces
        >>> eth0.iface
        'eth0'
        >>> eth0.ifname
        'eth0'
        >>> eth0.data['max'].rx # Old-style access
        2047
        >>> eth0.max.rx # New-style access
        2047

    """

    Parameters = namedtuple("Parameters", ["rx", "rx_mini", "rx_jumbo", "tx"])

    @property
    def ifname(self):
        """
        Return the name of network interface in content.
        """
        return self.iface

    def parse_content(self, content):
        """
        Parse ``ethtool -g`` info into a dictionary.
        """
        self.data = {}
        # First guess from path information
        self.iface = extract_iface_name_from_path(self.file_path, "ethtool_-g_")
        self.max = self.current = None
        if "ethtool" in content[0]:
            return

        if "Cannot get" in content[0]:
            # cannot got pause param in ethtool
            self.iface = extract_iface_name_from_content(content[1])
            return

        self.iface = extract_iface_name_from_content(content[0])

        def set_section(section, data):
            if section:
                ringdata = Ring.Parameters(**section_data)
                setattr(self, section, ringdata)
                self.data[section] = ringdata

        section = None
        sections = {'Pre-set maximums:': 'max', 'Current hardware settings:': 'current'}
        section_data = {}
        # Skip "Ring parameters for interface:"
        for line in content[1:]:
            if line in sections:
                set_section(section, section_data)
                section = sections[line]
                section_data = {}
            elif ':' in line:
                # key: value, store in section data for now
                key, value = (s.strip() for s in line.split(":", 1))
                section_data[key.replace(" ", "_").lower()] = int(value)

        # Handle last found section, if any
        set_section(section, section_data)


@parser(Specs.ethtool_S)
class Statistics(CommandParser):
    """
    Parse information for the ``ethtool -S`` command.

    All values are made available as keys in the ``data`` dictionary, and as
    properties - e.g. ``obj.data['rx_jabbers']`` is available as
    ``obj.rx_jabbers``.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.

    Sample partial input for ``/sbin/ethtool -S eth0``::

        NIC statistics:
             rx_octets: 808488730
             rx_fragments: 0
             rx_ucast_packets: 1510830
             rx_mcast_packets: 678653
             rx_bcast_packets: 9921
             rx_fcs_errors: 0
             rx_align_errors: 0
             rx_xon_pause_rcvd: 0
             rx_xoff_pause_rcvd: 0
             rx_mac_ctrl_rcvd: 0
             rx_xoff_entered: 0
             rx_frame_too_long_errors: 0
             rx_jabbers: 0

    Examples:
        >>> len(stats) # All interfaces in a list
        1
        >>> type(stats[0])
        <class 'insights.parsers.ethtool.Statistics'>
        >>> eth0 = stats[0] # Would normally iterate through interfaces
        >>> eth0.iface
        'eth0'
        >>> eth0.ifname
        'eth0'
        >>> eth0.data['rx_octets']  # Data as integers
        808488730
        >>> eth0.data['rx_fcs_errors']
        0
    """

    @property
    def ifname(self):
        """
        Return the name of network interface in content.
        """
        return self.iface

    def search(self, pattern, flags=0):
        """
        Finds all the parameters matching a given regular expression.

        Parameters:
            pattern (raw): A regular expression
            flags (int): Regular expression flags summed from ``re`` constants.

        Returns:
            (dict): A dictionary of the key/value pairs where the key matches
            the given regular expression.  An empty dictionary is returned if
            no keys matched.
        """
        results = {}
        for k, v in self.data.items():
            if re.search(pattern, k, flags):
                results[k] = v
        return results

    def parse_content(self, content):
        '''
        Parse the output of ``ethtool -S``.
        '''
        self.data = {}
        self.iface = extract_iface_name_from_path(self.file_path, "ethtool_-S_")

        if "NIC statistics:" not in content[0]:
            # if there's no data, then return self.data immediately.
            # in this situation, content may looks like:
            # "no stats available" or
            # "Cannot get stats strings self.datarmation: Operation not supported"
            return

        for line in content[1:]:  # ignore 'NIC statistics' line
            if ':' not in line:
                continue
            # Some interfaces can report keys with colons in them, e.g.
            # "rxq0: rx_pkts", so look for the last colon and strip from
            # there.
            i = line.rfind(':')
            key = line[:i].strip()
            value = line[i + 2:].strip()
            value = int(value)
            self.data[key] = value


@parser(Specs.ethtool)
class Ethtool(CommandParser):
    """
    Parses output of ``ethtool`` command.

    Raises:
        ParseException: Raised when any problem parsing the command output.

    Attributes:
        data (dict): Dictionary of keys with values in a list.
        iface (str): Interface name.
        supported_link_modes (list): A list of the 'Supported link modes'
            values, split into individual words.
        available_link_modes (list): A list of the 'Available link modes'
            values, split into individual words.
        supported_link_modes (list): A list of the 'Supported ports' values,
            split into individual words.

    Sample input::

        Settings for eth0:
                Supported ports: [ TP MII ]
                Supported link modes:   10baseT/Half 10baseT/Full
                                       100baseT/Half 100baseT/Full
                Supported pause frame use: No
                Supports auto-negotiation: Yes
                Advertised link modes:  10baseT/Half 10baseT/Full
                                        100baseT/Half 100baseT/Full
                Advertised pause frame use: Symmetric
                Advertised auto-negotiation: Yes
                Link partner advertised link modes:  10baseT/Half 10baseT/Full
                                                     100baseT/Half 100baseT/Full
                Link partner advertised pause frame use: Symmetric
                Link partner advertised auto-negotiation: No
                Speed: 100Mb/s
                Duplex: Full
                Port: MII
                PHYAD: 32
                Transceiver: internal
                Auto-negotiation: on
        Cannot get wake-on-lan settings: Operation not permitted
                Current message level: 0x00000007 (7)
                                       drv probe link
        Cannot get link status: Operation not permitted

    For historic reasons, values drawn from the data are stored as lists,
    with each item being the value on one line.

    Examples:
        >>> len(ethers) # All interfaces in a list
        1
        >>> type(ethers[0])
        <class 'insights.parsers.ethtool.Ethtool'>
        >>> ethinfo = ethers[0] # Would normally iterate through interfaces
        >>> ethinfo.ifname
        'eth0'
        >>> ethinfo.speed
        ['100Mb/s']
        >>> ethinfo.link_detected
        False
        >>> 'Cannot get link status' in ethinfo.data
        True
        >>> ethinfo.data['Cannot get link status']  # Dictionary for all data
        ['Operation not permitted']
        >>> ethinfo.data['Supported pause frame use']
        ['No']
        >>> ethinfo.data['PHYAD']  # Values as lists of strings for historic reasons
        ['32']
        >>> ethinfo.supported_link_modes  # This is collected across multiple lines and split
        ['10baseT/Half', '10baseT/Full', '100baseT/Half', '100baseT/Full']
        >>> ethinfo.advertised_link_modes
        ['10baseT/Half', '10baseT/Full', '100baseT/Half', '100baseT/Full']
        >>> ethinfo.supported_ports  # This is converted to a list of strings
        ['TP', 'MII']
    """
    @property
    def ifname(self):
        """str: Return the name of network interface in content."""
        return self.iface

    @property
    def speed(self):
        """list (str): Return field in Speed."""
        return self.data.get('Speed')

    @property
    def link_detected(self):
        """boolean: Returns field in Link detected."""
        return self.data.get('Link detected', ['no'])[0] == 'yes'

    def parse_content(self, content):
        self.data = {}
        self.iface = self.file_name.replace("ethtool_", "")

        # One way to get this result is run command '$ ethtool'.
        if "ethtool: bad command line argument(s)" in content[0]:
            raise ParseException('ethtool: bad command line argument for ethtool', content)

        if "Settings for" not in content[0]:
            raise ParseException("ethtool: unrecognised first line '{l}'".format(l=content[0]))

        self.data['ETHNIC'] = content[0].split()[-1].strip(':')

        if "No data available" in content[1]:
            raise ParseException('Fake ethnic as ethtool command argument', content)

        key = value = None
        for line in content[1:]:
            line = line.strip()
            if line:
                try:
                    if ':' in line:
                        key, value = line.split(':', 1)
                        key = key.strip()
                        self.data[key] = [value.strip()]
                    else:
                        self.data[key].append(line)
                except:
                    raise ParseException('Ethtool unable to parse content', line)

        self.supported_link_modes = []
        if 'Supported link modes' in self.data:
            for pair in self.data['Supported link modes']:
                self.supported_link_modes += pair.split()

        self.advertised_link_modes = []
        if 'Advertised link modes' in self.data:
            for pair in self.data['Advertised link modes']:
                self.advertised_link_modes += pair.split()

        self.supported_ports = []
        if 'Supported ports' in self.data:
            self.supported_ports = self.data['Supported ports'][0].strip('[] ').split()



from insights.parsers import autofs_conf
from insights.tests import context_wrap

AUTOFS_CONF = """
#
# Define default options for autofs.
# Heavily edited for brevity by removing most of the useless comments
#
[ autofs ]
#
# timeout - set the default mount timeout in secons. The internal
#	    program default is 10 minutes, but the default installed
#	    configuration overrides this and sets the timeout to 5
#	    minutes to be consistent with earlier autofs releases.
#
timeout = 300
#
# browse_mode - maps are browsable by default.
#
browse_mode = no
#
# mount_nfs_default_protocol - specify the default protocol used by
# 			       mount.nfs(8). Since we can't identify
# 			       the default automatically we need to
# 			       set it in our configuration.
#
#mount_nfs_default_protocol = 3
mount_nfs_default_protocol = 4
#
# Define global options for the amd parser within autofs.
#
[ amd ]
#
# Override the internal default with the same timeout that
# is used by the override in the autofs configuration, sanity
# only change.
#
dismount_interval = 300
#
# map_type = file
#
"""


class TestAutoFSConf():
    def test_standard_autofs_conf(self):
        cfg = autofs_conf.AutoFSConf(context_wrap(AUTOFS_CONF))

        assert cfg.get(" autofs ", "timeout") == '300'
        assert cfg.get(" autofs ", "browse_mode") == 'no'
        assert cfg.get(" autofs ", "mount_nfs_default_protocol") == '4'
        assert cfg.get(" amd ", "dismount_interval") == '300'

        # Check that things set in comments do not appear
        assert not cfg.has_option(" amd ", "map_type")
        # Check that nonexistent sections do not appear
        assert "nfs" not in cfg



from __future__ import print_function
import six
import sys
import inspect
from collections import namedtuple

from pprint import pprint
from insights import dr, datasource, rule, condition, incident, parser
from insights.core.context import ExecutionContext
from insights.formats import Formatter, FormatterAdapter, render


try:
    from colorama import Fore, Style, init
    init()
except ImportError:
    print("Install colorama if console colors are preferred.")

    class Default(type):
        def __getattr__(*args):
            return ""

    class Fore(six.with_metaclass(Default)):
        pass

    class Style(six.with_metaclass(Default)):
        pass


def _find_context(broker):
    for k, v in broker.instances.items():
        if inspect.isclass(k) and issubclass(k, ExecutionContext):
            return v


class HumanReadableFormat(Formatter):
    """
    This class prints a human readable summary of rule hits. It should be used
    as a context manager and given an instance of an
    ``insights.core.dr.Broker``. ``dr.run`` should be called within the context
    using the same broker.

    Args:
        broker (Broker): the broker to watch and provide a summary about.
        missing (bool): shows rules with missing dependencies if True.
        tracebacks (bool): shows tracebacks if any exceptions were raised.
            Useful for debugging.
        dropped (bool): Shows any files that weren't collected if running
            against an archive. Included for a corner case and typically not
            used in general.
        stream (file-like): Output is written to stream. Defaults to sys.stdout.
    """
    def __init__(self, broker,
            missing=False,
            tracebacks=False,
            dropped=False,
            fail_only=False,
            stream=sys.stdout):
        self.broker = broker
        self.missing = missing
        self.tracebacks = tracebacks
        self.dropped = dropped
        self.fail_only = fail_only
        self.stream = stream

    def print_header(self, header, color):
        ln = len(header)
        print(color + '-' * ln, file=self.stream)
        print(header, file=self.stream)
        print('-' * ln + Style.RESET_ALL, file=self.stream)

    def preprocess(self):
        response = namedtuple('response', 'color label intl title')
        self.responses = {'skip': response(color=Fore.BLUE, label="SKIP", intl='S', title="Missing Deps: "),
                          'pass': response(color=Fore.GREEN, label="PASS", intl='P', title="Passed      : "),
                          'fingerprint': response(color=Fore.YELLOW, label="FINGERPRINT", intl='P',
                                                  title="Fingerprint : "),
                          'rule': response(color=Fore.RED, label="FAIL", intl='F', title="Failed      : "),
                          'metadata': response(color=Fore.YELLOW, label="META", intl='M', title="Metadata    : "),
                          'metadata_key': response(color=Fore.MAGENTA, label="META", intl='K', title="Metadata Key: "),
                          'exception': response(color=Fore.RED, label="EXCEPT", intl='E', title="Exceptions  : ")
                          }

        self.counts = {}
        for key in self.responses:
            self.counts[key] = 0

        self.print_header("Progress:", Fore.CYAN)
        self.broker.add_observer(self.progress_bar, rule)
        self.broker.add_observer(self.progress_bar, condition)
        self.broker.add_observer(self.progress_bar, incident)
        self.broker.add_observer(self.progress_bar, parser)

    def progress_bar(self, c, broker):
        """
        Print the formated progress information for the processed return types
        """
        v = broker.get(c)

        if v and isinstance(v, dict) and len(v) > 0 and 'type' in v:
            if v["type"] in self.responses:
                print(self.responses[v["type"]].color + self.responses[v["type"]].intl + Style.RESET_ALL, end="", file=self.stream)
            else:
                print(".", end="", file=self.stream)
        elif c in broker.exceptions:
            self.counts['exception'] += len(broker.exceptions[c])
            print(Fore.RED + "E" + Style.RESET_ALL, end="", file=self.stream)
        return self

    def show_tracebacks(self):
        """ Show tracebacks """
        if self.broker.tracebacks:
            print(file=self.stream)
            print("Tracebacks:", file=self.stream)
            for t in self.broker.tracebacks.values():
                print(t, file=self.stream)

    def show_dropped(self):
        """ Show dropped files """
        ctx = _find_context(self.broker)
        if ctx and ctx.all_files:
            ds = self.broker.get_by_type(datasource)
            vals = []
            for v in ds.values():
                if isinstance(v, list):
                    vals.extend(d.path for d in v)
                else:
                    vals.append(v.path)
            dropped = set(ctx.all_files) - set(vals)
            pprint("Dropped Files:", stream=self.stream)
            pprint(dropped, indent=4, stream=self.stream)

    def show_description(self):
        """ Prints the formatted response for the matching return type """

        def printit(c, v):
            underline = "-" * len(dr.get_name(c))
            resp = self.responses[v["type"]]
            name = "%s[%s] %s%s" % (resp.color, resp.label, dr.get_name(c), Style.RESET_ALL)
            print(name, file=self.stream)
            print(underline, file=self.stream)
            print(render(c, v), file=self.stream)
            print(file=self.stream)

        for c in sorted(self.broker.get_by_type(rule), key=dr.get_name):
            v = self.broker[c]
            _type = v.get('type')
            if _type in self.responses:
                self.counts[_type] += 1
            if (_type and ((self.fail_only and _type == 'rule') or
                           ((self.missing and _type == 'skip') or
                            (not self.fail_only and _type != 'skip')))):
                printit(c, v)
        print(file=self.stream)

        self.print_header("Rule Execution Summary", Fore.CYAN)
        for c in self.counts:
            print(self.responses[c].color + self.responses[c].title + str(self.counts[c]) + Style.RESET_ALL, file=self.stream)

    def postprocess(self):
        if self.tracebacks:
            self.show_tracebacks()
        if self.dropped:
            self.show_dropped()

        print(file=self.stream)
        print(file=self.stream)
        self.print_header("Rules Executed", Fore.CYAN)
        self.show_description()


class HumanReadableFormatAdapter(FormatterAdapter):
    """ Displays results in a human readable format. """

    @staticmethod
    def configure(p):
        p.add_argument("-m", "--missing", help="Show missing requirements.", action="store_true")
        p.add_argument("-t", "--tracebacks", help="Show stack traces.", action="store_true")
        p.add_argument("-d", "--dropped", help="Show collected files that weren't processed.", action="store_true")
        p.add_argument("-F", "--fail-only", help="Show FAIL results only. Conflict with '-m' or '-f', will be dropped when using them together", action="store_true")

    def __init__(self, args):
        self.missing = args.missing
        self.tracebacks = args.tracebacks
        self.dropped = args.dropped
        self.fail_only = args.fail_only
        self.formatter = None
        if self.missing and self.fail_only:
            print(Fore.YELLOW + 'Options conflict: -m and -F, drops -F', file=sys.stderr)
            self.fail_only = False

    def preprocess(self, broker):
        self.formatter = HumanReadableFormat(broker,
                self.missing, self.tracebacks, self.dropped, self.fail_only)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()



import pytest

from insights.parsers.systool import SystoolSCSIBus, ParseException
from insights.tests import context_wrap

SYSTOOL_B_SCSI_V_1 = """
Bus = "scsi"

  Device = "1:0:0:0"
  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0"
    delete              = <store method only>
    device_blocked      = "0"
    device_busy         = "0"
    dh_state            = "detached"
    eh_timeout          = "10"
    evt_capacity_change_reported= "0"
    evt_inquiry_change_reported= "0"
    evt_lun_change_reported= "0"
    evt_media_change    = "0"
    evt_mode_parameter_change_reported= "0"
    evt_soft_threshold_reached= "0"
    iocounterbits       = "32"
    iodone_cnt          = "0x15b"
    ioerr_cnt           = "0x3"
    iorequest_cnt       = "0x16c"
    modalias            = "scsi:t-0x05"
    model               = "CD-ROM          "
    queue_depth         = "1"
    queue_type          = "none"
    rescan              = <store method only>
    rev                 = "1.0 "
    scsi_level          = "6"
    state               = "running"
    timeout             = "30"
    type                = "5"
    uevent              = "DEVTYPE=scsi_device
DRIVER=sr
MODALIAS=scsi:t-0x05"
    unpriv_sgio         = "0"
    vendor              = "VBOX    "

  Device = "2:0:0:0"
  Device path = "/sys/devices/pci0000:00/0000:00:0d.0/ata3/host2/target2:0:0/2:0:0:0"
    delete              = <store method only>
    device_blocked      = "0"
    device_busy         = "0"
    dh_state            = "detached"
    eh_timeout          = "10"
    evt_capacity_change_reported= "0"
    evt_inquiry_change_reported= "0"
    evt_lun_change_reported= "0"
    evt_media_change    = "0"
    evt_mode_parameter_change_reported= "0"
    evt_soft_threshold_reached= "0"
    iocounterbits       = "32"
    iodone_cnt          = "0x208c"
    ioerr_cnt           = "0x2"
    iorequest_cnt       = "0x20a6"
    modalias            = "scsi:t-0x00"
    model               = "VBOX HARDDISK   "
    queue_depth         = "31"
    queue_ramp_up_period= "120000"
    queue_type          = "simple"
    rescan              = <store method only>
    rev                 = "1.0 "
    scsi_level          = "6"
    state               = "running"
    timeout             = "30"
    type                = "0"
    uevent              = "DEVTYPE=scsi_device
DRIVER=sd
MODALIAS=scsi:t-0x00"
    unpriv_sgio         = "0"
    vendor              = "ATA     "
    vpd_pg80            =
    vpd_pg83            =

  Device = "host0"
  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata1/host0"
    uevent              = "DEVTYPE=scsi_host"

  Device = "host1"
  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata2/host1"
    uevent              = "DEVTYPE=scsi_host"

  Device = "host2"
  Device path = "/sys/devices/pci0000:00/0000:00:0d.0/ata3/host2"
    uevent              = "DEVTYPE=scsi_host"

  Device = "target1:0:0"
  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0"
    uevent              = "DEVTYPE=scsi_target"

  Device = "target2:0:0"
  Device path = "/sys/devices/pci0000:00/0000:00:0d.0/ata3/host2/target2:0:0"
    uevent              = "DEVTYPE=scsi_target"

"""

SYSTOOL_B_SCSI_V_ILLEGAL_2 = """
Bus = "scsi"

  Device = "1:0:0:0"
  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0"
    delete              = <store method only>
    type                is "5"
    uevent              = "DEVTYPE=scsi_device
DRIVER=sr
MODALIAS=scsi:t-0x05"
    unpriv_sgio         = "0"
    vendor              = "VBOX    "
"""

SYSTOOL_B_SCSI_V_ILLEGAL_3 = """
Bus = "scsi"

  Device = "1:0:0:0"
  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0"
    delete              = <store method only>
                        = "5"
    uevent              = "DEVTYPE=scsi_device
DRIVER=sr
MODALIAS=scsi:t-0x05"
    unpriv_sgio         = "0"
    vendor              = "VBOX    "
"""

SYSTOOL_B_SCSI_V_ILLEGAL_4 = """
Bus = "scsi"

  Device path = "/sys/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0"
    delete              = <store method only>
    type                = "5"
    uevent              = "DEVTYPE=scsi_device
DRIVER=sr
MODALIAS=scsi:t-0x05"
    unpriv_sgio         = "0"
    vendor              = "VBOX    "
"""

SYSTOOL_B_SCSI_V_ILLEGAL_5 = """
Bus = "scsi"

"""

SYSTOOL_B_SCSI_V_ILLEGAL_6 = """
Bus = "abc"

"""


def test_systoolscsibus_with_legal_input():
    res = SystoolSCSIBus(context_wrap(SYSTOOL_B_SCSI_V_1))

    assert len(res.data) == 7
    assert len(res.devices[0]) == 30
    assert len(res.devices[1]) == 33
    assert res.data['1:0:0:0']['uevent'] == 'DEVTYPE=scsi_device DRIVER=sr MODALIAS=scsi:t-0x05'
    assert res.data['1:0:0:0']['delete'] == '<store method only>'
    assert res.devices[1].get('vpd_pg83') == ''
    assert res.devices[1].get('not_exist') is None

    assert all([d.get('Device path') for d in res.devices])

    assert res.get_device_state('2:0:0:0') == 'running'
    assert res.get_device_state('host2') is None
    assert res.get_device_state('host_what') is None


def test_systoolscsibus_with_illegal_input():
    with pytest.raises(ParseException) as e_info:
        SystoolSCSIBus(context_wrap(SYSTOOL_B_SCSI_V_ILLEGAL_2))
    assert "Unparseable line without = " in str(e_info.value)

    with pytest.raises(ParseException) as e_info:
        SystoolSCSIBus(context_wrap(SYSTOOL_B_SCSI_V_ILLEGAL_3))
    assert "Unparseable line without key" in str(e_info.value)

    with pytest.raises(ParseException) as e_info:
        SystoolSCSIBus(context_wrap(SYSTOOL_B_SCSI_V_ILLEGAL_4))
    assert "Parsing Error for no heading Device-name" in str(e_info.value)

    with pytest.raises(ParseException) as e_info:
        SystoolSCSIBus(context_wrap(SYSTOOL_B_SCSI_V_ILLEGAL_5))
    assert "Parsing Error for this almost empty input." in str(e_info.value)

    with pytest.raises(ParseException) as e_info:
        SystoolSCSIBus(context_wrap(SYSTOOL_B_SCSI_V_ILLEGAL_6))
    assert "Unparseable first line of input:" in str(e_info.value)



'''
NeutronMetadataAgentIni - file ``/etc/neutron/metadata_agent.ini``
==================================================================

The ``NeutronMetadataAgentIni`` class parses the metadata-agent configuration file.
See the ``IniConfigFile`` class for more usage information.
'''

from .. import add_filter, parser, IniConfigFile
from insights.specs import Specs

add_filter(Specs.neutron_metadata_agent_ini, ["["])


@parser(Specs.neutron_metadata_agent_ini)
class NeutronMetadataAgentIni(IniConfigFile):
    """
    Parse the ``/etc/neutron/metadata_agent.ini`` configuration file.

    Sample configuration::

        [DEFAULT]
        # Show debugging output in log (sets DEBUG log level output)
        # debug = True

        # The Neutron user information for accessing the Neutron API.
        auth_url = http://localhost:5000/v2.0
        auth_region = RegionOne
        # Turn off verification of the certificate for ssl
        # auth_insecure = False
        # Certificate Authority public key (CA cert) file for ssl
        # auth_ca_cert =
        admin_tenant_name = %SERVICE_TENANT_NAME%
        admin_user = %SERVICE_USER%
        admin_password = %SERVICE_PASSWORD%

        # Network service endpoint type to pull from the keystone catalog
        # endpoint_type = adminURL

        # IP address used by Nova metadata server
        # nova_metadata_ip = 127.0.0.1

        # TCP Port used by Nova metadata server
        # nova_metadata_port = 8775

        # Which protocol to use for requests to Nova metadata server, http or https
        # nova_metadata_protocol = http

        # Whether insecure SSL connection should be accepted for Nova metadata server
        # requests
        # nova_metadata_insecure = False

        # Client certificate for nova api, needed when nova api requires client
        # certificates
        # nova_client_cert =

        # Private key for nova client certificate
        # nova_client_priv_key =

        # When proxying metadata requests, Neutron signs the Instance-ID header with a
        # shared secret to prevent spoofing.  You may select any string for a secret,
        # but it must match here and in the configuration used by the Nova Metadata
        # Server. NOTE: Nova uses the same config key, but in [neutron] section.
        # metadata_proxy_shared_secret =

        # Location of Metadata Proxy UNIX domain socket
        # metadata_proxy_socket = $state_path/metadata_proxy

        # Metadata Proxy UNIX domain socket mode, 3 values allowed:
        # 'deduce': deduce mode from metadata_proxy_user/group values,
        # 'user': set metadata proxy socket mode to 0o644, to use when
        # metadata_proxy_user is agent effective user or root,
        # 'group': set metadata proxy socket mode to 0o664, to use when
        # metadata_proxy_group is agent effective group,
        # 'all': set metadata proxy socket mode to 0o666, to use otherwise.
        # metadata_proxy_socket_mode = deduce

        # Number of separate worker processes for metadata server. Defaults to
        # half the number of CPU cores
        # metadata_workers =

        # Number of backlog requests to configure the metadata server socket with
        # metadata_backlog = 4096

        # URL to connect to the cache backend.
        # default_ttl=0 parameter will cause cache entries to never expire.
        # Otherwise default_ttl specifies time in seconds a cache entry is valid for.
        # No cache is used in case no value is passed.
        # cache_url = memory://?default_ttl=5

        [AGENT]
        # Log agent heartbeats from this Metadata agent
        # log_agent_heartbeats = False


    Examples:

        >>> metadata_agent_ini.has_option('AGENT', 'log_agent_heartbeats')
        True
        >>> metadata_agent_ini.get("DEFAULT", "auth_url") == 'http://localhost:35357/v2.0'
        True
        >>> metadata_agent_ini.getint("DEFAULT", "metadata_backlog")
        4096

    """
    pass



"""
Satellite Version
=================

Combiner for Satellite Version information. It uses the results of
the ``Satellite6Version`` parser and the ``InstalledRpms`` parser to determine
the satellite version

"""

from .. import combiner, SkipComponent
from ..parsers.satellite_version import Satellite6Version as Sat6Ver
from ..parsers.installed_rpms import InstalledRpms


# Only track 6.0.x and 6.1.x. See https://access.redhat.com/articles/1343683
# But, there are some mistakes in the KCS, the versions in below map
# are corrected according to the corresponding ERRATA pages.
sat6_ver_map = {
        # Sat     foreman     candlepin   katello
        '6.0.8': ('1.6.0.53', '0.9.23', '1.5.0'),
        '6.1.1': ('1.7.2.33', '0.9.49.3', '2.2.0.14'),
        '6.1.2': ('1.7.2.36', '0.9.49.6', '2.2.0.16'),
        '6.1.3': ('1.7.2.43', '0.9.49.8', '2.2.0.16'),
        '6.1.4': ('1.7.2.46', '0.9.49.9', '2.2.0.16'),
        '6.1.5': ('1.7.2.49', '0.9.49.9', '2.2.0.16'),
        '6.1.6': ('1.7.2.50', '0.9.49.9', '2.2.0.17'),
        '6.1.7': ('1.7.2.53', '0.9.49.11', '2.2.0.17'),
        '6.1.8': ('1.7.2.55', '0.9.49.12', '2.2.0.19'),
        '6.1.9': ('1.7.2.56', '0.9.49.12', '2.2.0.19'),
        '6.1.10': ('1.7.2.61', '0.9.49.16', '2.2.0.19'),
        '6.1.11': ('1.7.2.62', '0.9.49.19', '2.2.0.19'),
        '6.1.12': ('1.7.2.63', '0.9.49.23', '2.2.0.19'),
}


def _parse_sat_version(version):
    ver_sp = version.split(".") if version else []
    major = int(ver_sp[0]) if ver_sp and ver_sp[0].isdigit() else None
    minor = int(ver_sp[1]) if len(ver_sp) > 1 and ver_sp[1].isdigit() else None
    return [major, minor]


@combiner([Sat6Ver, InstalledRpms])
class SatelliteVersion(object):
    """
    Check the parsers
    :class:`insights.parsers.satellite_version.Satellite6Version` and
    :class:`insights.parsers.installed_rpms.InstalledRpms` for satellite version
    information.

    Below is the logic to determine the satellite version::

        1. For Satellite 6.1.x:
            a. Check the version information in below files at first
               - https://access.redhat.com/solutions/1392633

               File: /usr/share/foreman/lib/satellite/version.rb

            b. Check the version of package foreman, candlepin and katello,
               when the files listed in (a) are not found.
               - https://access.redhat.com/articles/1343683
                 NOTE: There are some mistakes in the KCS, the versions in this
                       combiner are corrected according to the corresponding
                       ERRATA.

                            Sat 6.0.8   Sat 6.1.10  Sat 6.1.11
               foreman      1.6.0.53    1.7.2.61    1.7.2.62
               candlepin    0.9.23      0.9.49.16   0.9.49.19
               katello      1.5.0       2.2.0       2.2.0

        2. For Satellite 6.2.x:
           - https://access.redhat.com/solutions/1392633

           Check the version of package `satellite` directly:

                            Sat 6.0.x   Sat 6.1.x   Sat 6.2.x
                satellite   -           -           6.2.x

        3. For Satellite 5.x
           - https://access.redhat.com/solutions/1224043
             NOTE: Because of `satellite-branding` is not deployed in Satellite
                   5.0~5.2, and `satellite-schema` can also be used for checking
                   the version, here checked `satellite-schema` instead of
                   `satellite-branding`.

           Check the version of package satellite-schema directly:

                                        Sat 5.0~5.2     Sat 5.3 ~
                rhn-satellite-schema    ok              -
                satellite-schema        -               ok

    Attributes:
        full(str): the full version format like `version-release`.
        version(str): the satellite version do not includes `release`.
        release(str): the `release` string in the version.
        major(int): the major version.
        minor(int): the minor version.

    Raises:
        SkipComponent: When it's not a Satellite machine or the Satellite
            version cannot be determined according to current information.
            However, this type of Exceptions will be ignored.

    Examples:
        >>> sat_ver = shared[satellite_version]
        >>> sat_ver.major
        6
        >>> sat_ver.minor
        2
        >>> sat_ver.version
        '6.2.2'
        >>> sat_ver.full
        '6.2.2.1-1.0.el7sat'
        >>> sat_ver.release
        '1.0.el7sat'

    """
    def __init__(self, sat6_ver, rpms):
        self.full = None
        self.version = None
        self.release = None
        self.major = None
        self.minor = None

        # For Satellite 6.1.x, if satellite_version/version.rb is available:
        if sat6_ver:
            # no 'release' in this situation
            self.full = sat6_ver.full
            self.version = sat6_ver.version
            self.major = sat6_ver.major
            self.minor = sat6_ver.minor
            return
        if rpms:
            # For Satellite 6.2.x, check the ``satellite`` package directly
            sat62_pkg = rpms.get_max('satellite')
            if sat62_pkg:
                self.full = sat62_pkg.package
                self.version = sat62_pkg.version
                self.release = sat62_pkg.release
                self.major, self.minor = _parse_sat_version(self.version)
                return
            # For Satellite 6.0.x/6.1.x, check the version of:
            # - foreman, candlepin and katello
            fman = rpms.get_max('foreman')
            cndp = rpms.get_max('candlepin')
            ktlo = rpms.get_max('katello')
            if fman and cndp and ktlo:
                for sat_ver, map_ver in sat6_ver_map.items():
                    if all(pkg.version.startswith(mv) for pkg, mv
                            in zip([fman, cndp, ktlo], map_ver)):
                        # no 'release' in this situation
                        self.major, self.minor = _parse_sat_version(sat_ver)
                        self.full = self.version = sat_ver
                        return
                # satellite packages conflict
                raise SkipComponent("RPMs conflict, unable to determine Satellite version")
            # For Satellite 5.x
            sat5_pkg = rpms.get_max('satellite-schema')
            sat5_pkg = sat5_pkg if sat5_pkg else rpms.get_max('rhn-satellite-schema')
            if sat5_pkg:
                self.full = sat5_pkg.package
                self.version = sat5_pkg.version
                self.release = sat5_pkg.release
                self.major, self.minor = _parse_sat_version(self.version)
                return
            raise SkipComponent("Not a Satellite machine")
        raise SkipComponent("No RPMs list, unable to determine Satellite version")
